; *****************************************************************************
; *****************************************************************************
;
;       File    : Data.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Data Segment / Reset Code
;       Created : 24th September 1997
;       Modified: 5th October 1997
;
;       Functions :
;               DReset          Reset the Data Area, CPU etc.
;
; *****************************************************************************
; *****************************************************************************


.386p 
.model flat
include extern.inc
.code

; *****************************************************************************
;
;                               Reset the system
;
; *****************************************************************************

DReset Proc
        push    eax
        push    ebx
        push    edx
        xor     eax,eax                 ; Clear the main area to all zero
        mov     ebx,offset DStartData
_DRS1:  mov     [ebx],eax
        add     ebx,4
        cmp     ebx,offset DEndData
        jb      _DRS1
        mov     [_SP],0F000h            ; Set SP = $F000, apparently default !
        mov     [_IFF1],1               ; Turn on Global Interrupt Enable
        mov     [_IFF2],1
        mov     [Border],0              ; nice border colour..

        mov     [FrmSkip],3             ; 1/3 Frame Skip Rate
        mov     [FrmRate],3             ; 3 frames per Clock Tick
        mov     [Issue2],0              ; Standard Spectrum
        mov     [Volume],15             ; Sound Volume

        call    NSoundOff               ; No Sound

        mov     ebx,offset Memory       ; Load Speccy ROM
        mov     edx,offset _DRSName
        call    FLoad

        mov     al,[byte ptr Memory+0]  ; for 16 bit I reads on $FFFF
        mov     [byte ptr Memory+10000h],al

        mov     [_RBit7],128

        mov     ebx,16
_DRS2:  mov     eax,[dword ptr _Msg+ebx]
        mov     [dword ptr Memory+0153Bh+ebx],eax
        sub     ebx,4
        jge     _DRS2

        pop     edx
        pop     ebx
        pop     eax
        ret

_Msg    db 'Paul Robson 199','7'+128,'    '
_DRSName db     'Spectrum.Rom',0        ; Alternate ROM name
        align   dword

EndP DReset

; *****************************************************************************
;
;                               System Data Area
;
; *****************************************************************************

.data?
align   dword

DStartData:

FrmSkip dd      ?                       ; Frame Skip Rate
FrmRate dd      ?                       ; Frames per PC Clock Tick
Issue2  dd      ?                       ; Non-Zero if Issue 2 Spectrum
 
Memory: db      65536 dup (?)           ; Z80 Memory Space
        dd      1024 dup (?)            ; 16 bit write to $FFFF

DbgData dw      ?,?                     ; Debug Data Pointer
Border  db      ?,?,?,?                 ; Border Colour (lower 3 bits)
Volume  db      ?,?,?,?                 ; Sound Volume
SoundMode dd    ?                       ; Sound Mode
Break   dw      ?,?                     ; Breakpoint

TAPPtr  dd      ?                       ; Pointer to next tape block

Scan    dw      ?,?                     ; Current Scanline (-1 if border)
Cycles  dd      ?                       ; Cycle Count
FrameCount dd   ?                       ; Frame Counter
Special dd      ?                       ; Set to non-zero if special keys...
ExecPtr dd      ?                       ; Vector for 'execute one command'

_AF:                                    ; Z80 Main Register Set
_F      db      ?                       ; 4 main register pairs
_A      db      ?,?,?
_BC:
_C      db      ?
_B      db      ?,?,?
_DE:
_E      db      ?
_D      db      ?,?,?
_HL:
_L      db      ?
_H      db      ?,?,?

_IX     dw      ?,?                     ; Index registers
_IY     dw      ?,?
_SP     dw      ?,?                     ; Stack pointer
_PC     dw      ?,?                     ; Program Counter

_XAF    dw      ?,?                     ; Alternate Register Set
_XBC    dw      ?,?
_XDE    dw      ?,?
_XHL    dw      ?,?

_IMode  db      ?,?,?,?                 ; Interrupt Mode (0,1,2)
_IFF1   db      ?,?,?,?                 ; Interrupt enable flags
_IFF2   db      ?,?,?,?
_I      db      ?,?,?,?                 ; I Register
_R      db      ?,?,?,?                 ; R Register : bit 7 isn't masked.
                                        ; until LD A,R.
_RBit7  db      ?,?,?,?

DEndData:

end
