; *****************************************************************************
; *****************************************************************************
;
;       File    : Debug.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Debugger
;       Created : 24th September 1997
;       Modified: 12th October 1997
;
;       Functions :
;               BDebug          Run the Debugger
;               BDisplay        Display Debug Information Screen
;               BDisassemble    Disassemble Instruction
;               BSetExecute     Set the Execution Vector
;
; *****************************************************************************
; *****************************************************************************

.386p
.model flat
include extern.inc
.data
SPBreak dw      0,0
WorkPC  dw      0,0
.code

; *****************************************************************************
;
;                                Run the Debugger
;
; *****************************************************************************

BDebug Proc
        mov     ax,[word ptr _PC]
        mov     [WorkPC],ax
        call    NSoundOff               ; no sound
_BDB0:  call    BDisplay                ; main debugging loop
        mov     ebx,offset _BDBKey      ; get a keystroke
        call    KGet
        cmp     al,16                   ; if 0-15 modify.
        jb      _BDB1
        cmp     al,17                   ; 17 (F7) is step over
        je      _BDB4
        cmp     al,18                   ; 18 (F8) is Single Step
        je      _BDB2
        cmp     al,19                   ; 19 (F9) run to break
        je      _BDB3
        cmp     al,20                   ; 20 (Esc) run
        je      _BDB5
        cmp     al,21                   ; 21 (F1) Set Breakpoint
        je      _BDB13

        cmp     al,22                   ; 22 (Up/Down) Change Value
        jae     _BDB1
        cmp     al,16                   ; if not display screen (key 16)
        jne     _BDB0                   ; get another instruction

        call    ScRebuild
        mov     ebx,offset _BDBKey
        call    KGet
        jmp     _BDB0

_BDB5:  mov     al,dbmRun               ; switch to run mode
        call    BSetExecute             ; set the execution mode
        jmp     _BDBX
_BDBX:  ret

_BDB13: mov     ax,[WorkPC]             ; F1 (Set Breakpoint)
        mov     [Break],ax
        jmp     _BDB0

_BDB1:  mov     esi,offset WorkPC       ; modify PC Ctrl:Data Alt:Break
        test    [byte ptr KeyTable+kCtrl],128
        jz      _BDB11
        mov     esi,offset DbgData
_BDB11: test    [byte ptr KeyTable+kAlt],128
        jz      _BDB12
        mov     esi,offset Break
_BDB12: cmp     al,22                   ; is it cursor movement
        jae     _BDB20
        mov     bx,[esi]                ; do the rotate-in thing
        shl     bx,4
        or      bl,al
        mov     [esi],bx
        jmp     _BDB0                   ; go back...

_BDB20: cmp     al,22                   ; Cursor movement on ESI
        je      _BDB22
        cmp     al,23
        je      _BDB23
        cmp     al,24
        je      _BDB24
        cmp     al,25
        je      _BDB25
        cmp     al,26
        je      _BDB26
        jmp     _BDB0
_BDB22: dec     [word ptr 0+esi]
        jmp     _BDB0
_BDB23: inc     [word ptr 0+esi]
        jmp     _BDB0
_BDB24: sub     [word ptr 0+esi],16
        jmp     _BDB0
_BDB25: add     [word ptr 0+esi],16
        jmp     _BDB0
_BDB26: mov     ax,[_PC]
        mov     [esi],ax
        jmp     _BDB0


_BDB2:  mov     al,dbmSingleStep        ; Single Step
        call    BSetExecute
        ret

_BDB3:  mov     al,dbmRunBreak          ; Run to Break
        call    BSetExecute
        ret

_BDB4:  mov     ax,[word ptr _SP]
        mov     [word ptr SPBreak],ax
        mov     al,dbmStepOver
        call    BSetExecute
        ret

_BDBKey:db      k0,k1,k2,k3,k4,k5,k6,k7 ; 0-7           keys 0-7
        db      k8,k9,kA,kB,kC,kD,kE,kF ; 8-15          keys 8,9,A-F
        db      kF5                     ; 16            F5 (display screen)
        db      kF7                     ; 17            F7 (step through)
        db      kF8                     ; 18            F8 (single step)
        db      kF9                     ; 19            F9 (Run to break)
        db      kEsc                    ; 20            ESC (run normally)
        db      kF1                     ; 21            F1 (Set Breakpoint)
        db      kUp,kDown               ; 22,23         Increment/Decrement
        db      kPageUp,kPageDn,kHome   ; 24,25,26      Up/Down/To PC
        db      0
                               
EndP BDebug

; *****************************************************************************
;
;            Set execution mode to AL (must be one of the dbm*)
;
; *****************************************************************************

BSetExecute Proc
        and     eax,7
        mov     eax,[dword ptr BSETable+eax*4]
        mov     [ExecPtr],eax
        ret
BSETable:dd     Xecute,BSingleStep,BRunBreak,BStepOver
EndP BSetExecute

; *****************************************************************************
;
;          Step over - run until breakpoint OR Stack returned
;
; *****************************************************************************

BStepOver Proc
        call    Xecute
        mov     ax,[word ptr _SP]
        cmp     ax,[word ptr SPBreak]
        je      _SBreak
        mov     ax,[word ptr _PC]
        cmp     ax,[word ptr Break]
        je      _SBreak
        ret
_SBreak:call    BDebug
        ret
EndP BStepOver

; *****************************************************************************
;
;                               Run to breakpoint
;
; *****************************************************************************

BRunBreak Proc
        mov     ax,[word ptr _PC]
        cmp     ax,[word ptr Break]
        je      _Break
        call    Xecute
        ret
_Break: call    BDebug
        ret
EndP BRunBreak

; *****************************************************************************
;
;                                   Single Step
;
; *****************************************************************************

BSingleStep Proc
        call    Xecute
        call    BDebug
        ret
EndP BSingleStep

; *****************************************************************************
;
;                       Display the Debugging Screen
;
; *****************************************************************************

XD      equ     VCSetX+1                ; Position
YD      equ     VCSetY+2
CLABEL  equ     VCSetColour+2           ; Colour Scheme
CVALUE  equ     VCSetColour+11

BDisplay Proc
        push    eax
        push    ebx
        push    ebp
        and     [_R],07Fh               ; R is 7 bit only. LD A,R does it too.
        mov     al,[_RBit7]
        or      [_R],al
        mov     ebp,offset BDebugData
_BDS1:  mov     al,[ebp]                ; get next
        inc     ebp
        or      al,al                   ; exit if zero
        jz      _BDS2
        cmp     al,10                   ; 1-9 is display command
        jb      _BDS3
        call    VPrint
        jmp     _BDS1

_BDS3:  mov     ebx,[ebp]               ; get address to display
        add     ebp,4
        cmp     al,4
        je      _BDS11
        cmp     al,3
        je      _BDS13
        cmp     al,2
        je      _BDS4
        cmp     al,1
        je      _BDS5
        jmp     _BDS1
_BDS13: mov     al,[ebx]                ; print a Boolean
        or      al,al
        jz      _BDS14
        mov     al,1
_BDS14: add     al,48
        call    VPrint
        jmp     _BDS1
_BDS11: mov     ax,[ebx+2]              ; print a Dword
        call    VPrintWord
_BDS4:  mov     ax,[ebx]                ; print a word
        call    VPrintWord
        jmp     _BDS1
_BDS5:  mov     al,[ebx]                ; print a word
        call    VPrintByte
        jmp     _BDS1

_BDS2:  mov     al,XD+8                 ; Print Status
        call    VPrint
        mov     al,YD+1
        call    VPrint
        mov     al,CVALUE
        call    VPrint

        mov     bl,[_F]                 ; Get flag register
        mov     bh,8                    ; Counter
        mov     ebp,offset _BDFNM       ; pointer to flag names
_BDS6:  mov     al,[ebp]
        inc     ebp
        test    bl,128
        jnz     _BDS7
        mov     al,'.'
_BDS7:  call    VPrint
        add     bl,bl
        dec     bh
        jnz     _BDS6

        mov     ebp,[dword ptr WorkPC]  ; do the disassembly list
        mov     bh,YD                   ; bh = y position
_BDS8:  mov     al,XD+17                ; move cursor
        call    VPrint
        mov     al,bh
        call    VPrint
        mov     al,CLABEL
        call    VPrint
        mov     ax,bp
        call    VPrintWord
        mov     al,VCSetColour+12
        call    VPrint
        mov     al,' '
        cmp     bp,[word ptr _PC]
        jne     _BDS12
        mov     al,'>'
_BDS12: call    VPrint
        mov     al,CVALUE
        call    VPrint
        call    BDisassemble
        inc     bh
        cmp     bh,YD+17
        jne     _BDS8

        mov     al,VCSetX
        call    VPrint
        mov     al,YD+16
        call    VPrint
        mov     al,VCSetColour+14
        call    VPrint
        mov     bl,40
        mov     al,'['
_BDS20: call    VPrint
        dec     bl
        jnz     _BDS20
        mov     bl,YD
_BDS21: mov     al,bl
        call    VPrint
        mov     al,XD+16
        call    VPrint
        mov     al,']'
        call    VPrint
        inc     bl
        cmp     bl,YD+17
        jne     _BDS21
        mov     al,XD+16
        call    VPrint
        mov     al,'{'
        call    VPrint
        mov     ebp,[dword ptr DbgData] ; now display the data
        mov     bh,YD+17
_BDS9:  mov     al,bh
        call    VPrint
        mov     al,XD+3
        call    VPrint
        mov     al,CLABEL
        call    VPrint
        mov     ax,bp
        call    VPrintWord
        mov     al,':'
        call    VPrint
        mov     al,CVALUE
        call    VPrint
        mov     bl,8
_BDS10: mov     al,32
        call    VPrint
        mov     al,[byte ptr Memory+ebp]
        call    VPrintByte
        inc     bp
        dec     bl
        jnz     _BDS10
        inc     bh
        cmp     bh,YD+22
        jnz     _BDS9

        pop     ebp                     ; exit
        pop     ebx
        pop     eax
        ret

_BDFNM  db      'NZ1H1PSC'              ; flag names
BDebugData:
        db      VCSetBack+0,VCClear     ; Clear the screen
        db      VCSetX+14,VCSetY,VCSetColour+15,"Z80 Debugger"
        db      CLABEL,XD,YD,"A",XD+10,"(",XD+15,")",CVALUE,XD+5,1
        dd      _A
        db      XD+11,2
        dd      _XAF
        db      CLABEL,XD,YD+1,"F",CVALUE,XD+5,1
        dd      _F
        db      CLABEL,XD,YD+2,"BC",XD+10,"(",XD+15,")"
        db      CVALUE,XD+5,2
        dd      _BC
        db      XD+11,2
        dd      _XBC
        db      CLABEL,XD,YD+3,"DE",XD+10,"(",XD+15,")"
        db      CVALUE,XD+5,2
        dd      _DE
        db      XD+11,2
        dd      _XDE
        db      CLABEL,XD,YD+4,"HL",XD+10,"(",XD+15,")"
        db      CVALUE,XD+5,2
        dd      _HL
        db      XD+11,2
        dd      _XHL
        db      CLABEL,XD,YD+5,"IX",CVALUE,XD+5,2
        dd      _IX
        db      CLABEL,XD,YD+6,"IY",CVALUE,XD+5,2
        dd      _IY
        db      CLABEL,XD,YD+7,"SP",CVALUE,XD+5,2
        dd      _SP
        db      CLABEL,XD,YD+8,"PC",CVALUE,XD+5,2
        dd      _PC
        db      CLABEL,XD,YD+9,"IRQM",CVALUE,XD+5,1
        dd      _IMode
        db      CLABEL,XD,YD+10,"IFF",XD+5,"1:x 2:x",CVALUE,XD+7,3
        dd      _IFF1
        db      XD+11,3
        dd      _IFF2
        db      CLABEL,XD,YD+11,"I",CVALUE,XD+5,1
        dd      _I
        db      CLABEL,XD,YD+12,"R",CVALUE,XD+5,1
        dd      _R
        db      CLABEL,XD,YD+13,"Scan",CVALUE,XD+5,2
        dd      Scan
        db      CLABEL,XD,YD+14,"Brk",CVALUE,XD+5,2
        dd      Break
        db      CLABEL,XD,YD+15,"T-St",CVALUE,XD+5,4
        dd      Cycles
        db      0

EndP BDisplay

; *****************************************************************************
;
;       Disassemble Z80 Instruction at EBP (logical address), which
;       will point to the next instruction on return.
;
; *****************************************************************************

BDisassemble Proc
        push    eax
        push    ebx
        mov     ebx,offset Disassemble  ; ebx points to the table list
        mov     ebx,[ebx]
        call    BDisTable
        pop     ebx
        pop     eax
        ret
EndP BDisassemble

; *****************************************************************************
;
;       Disassemble instruction at BP using table at address EBX.
;
; *****************************************************************************

BDisTable Proc
        mov     al,[byte ptr Memory+ebp]; al is the opcode number
        inc     bp
        or      al,al                   ; skip sufficient operations
        jz      _BDT1
_BDT3:  mov     ah,[ebx]                ; skip one operation
        inc     ebx
        or      ah,ah
        jnz     _BDT3                   ; until zero found
        dec     al                      ; do it AL times.
        jnz     _BDT3

_BDT1:  mov     al,[ebx]                ; print the text out
        inc     ebx
        or      al,al
        jz      _BDT0
        cmp     al,'#'                  ; # is word value
        je      _BDT10
        cmp     al,'*'                  ; * is byte value
        je      _BDT11
        cmp     al,'@'                  ; @ is signed byte value
        je      _BDT12
        cmp     al,''                  ;  is extension
        je      _BDT13
        cmp     al,'%'                  ; % is Index Reference
        je      _BDT15
        cmp     al,'^'                  ; ^ is DD FD offset
        je      _BDT17
        call    VPrint                  ; print the character
        jmp     _BDT1

_Index  db      0                       ; Current Index Register
_Offset db      0                       ; offset for DD/FD CB
        align   dword

_BDT10: mov     ax,[word ptr Memory+ebp]; display word
        call    VPrintWord
        add     bp,2
        jmp     _BDT1

_BDT11: mov     al,[byte ptr Memory+ebp]; display byte
        call    VPrintByte
        inc     bp
        jmp     _BDT1

_BDT17: mov     ah,[_Offset]            ; display saved CB offset
        jmp     _BDT20

_BDT12: mov     ah,[byte ptr Memory+ebp]; display signed byte
        inc     bp
_BDT20: mov     al,'+'
        test    ah,128
        jz      _BDT19
        mov     al,'-'
        xor     ah,255
        inc     ah
_BDT19: call    VPrint
        mov     al,ah
        call    VPrintByte
        jmp     _BDT1

_BDT13: mov     al,[ebx]                ; go to different table
        cmp     al,'9'
        jbe     _BDT14
        mov     [_Index],al             ; if > '9' then setting index register
        inc     ebx                     ; for FD or DD
        mov     al,[ebx]
_BDT14: and     eax,15
        cmp     al,4                    ; table 4 (xD CB) has offset before
        jne     _BDT16                  ; the instruction.
        mov     bl,[byte ptr Memory+ebp]
        inc     bp
        mov     [_Offset],bl
                                        ; locate table and go there !
_BDT16: mov     ebx,[offset Disassemble+eax*4]
        call    BDisTable               ; use that one to disassemble
        jmp     _BDT0

_BDT15: mov     al,[_Index]             ; print index register
        call    VPrint
        jmp     _BDT1

_BDT0:  ret

EndP BDisTable
     
End
