; *****************************************************************************
; *****************************************************************************
;
;       File    : Display.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Render the Spectrum Display
;       Created : 7th October 1997
;       Modified: 10th October 1997
;
;       Functions :
;               SCRebuild               Rebuild the Entire Screen
;               SCTop/BottomBorder      Redraw Borders
;               SCLine                  Render screen line AL.
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

.data
BufferPtr       dd      0               ; Address of Video Buffer
.code

; *****************************************************************************
;
;          Initialise Sinclair Display. On entry EAX = VGA Address
;
; *****************************************************************************

SCInitialise Proc
        mov     [BufferPtr],eax         ; save the video memory address
        ret
EndP SCInitialise

; *****************************************************************************
;
;                        Rebuild the entire screen
;
; *****************************************************************************

SCRebuild Proc
        push    eax
        call    SCTopBorder             ; draw the top border
        mov     al,0                    ; draw 192 display lines
_SCR0:  call    SCLine
        inc     al
        cmp     al,192
        jnz     _SCR0
        call    SCBottomBorder          ; draw the bottom border
        pop     eax
        ret
EndP SCRebuild

; *****************************************************************************
;
;                           Draw the top border
;
; *****************************************************************************

SCTopBorder Proc
        push    eax
        push    ebx
        push    ecx
        mov     ebx,[BufferPtr]         ; address to write to
        call    SCGetBorderPattern      ; get picture pattern
        mov     ecx,320*4/4             ; dwords to write out
_SCTB0: mov     [ebx],eax
        add     ebx,4
        dec     ecx
        jnz     _SCTB0
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP SCTopBorder

; *****************************************************************************
;
;                          Draw the bottom border
;
; *****************************************************************************

SCBottomBorder Proc
        push    eax
        push    ebx
        push    ecx
        mov     ebx,[BufferPtr]         ; address to write to
        add     ebx,196*320
        call    SCGetBorderPattern      ; get pattern to write
        mov     ecx,320*4/4             ; number of dwords to write
_SCBB0: mov     [ebx],eax
        add     ebx,4
        dec     ecx
        jnz     _SCBB0
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP SCBottomBorder

; *****************************************************************************
;
;                    Get the pattern for border colour
;
; *****************************************************************************

SCGetBorderPattern Proc
        mov     al,[Border]             ; eax = border & 7
        and     eax,7                   ; look up in colour table
        mov     eax,[dword ptr SCColourTable+eax*4]
        ret
EndP SCGetBorderPattern

; *****************************************************************************
;
;                      Draw a whole screen line
;
; *****************************************************************************

SCLine Proc
        push    eax
        push    ebx
        push    ecx
        push    edx
        and     eax,0FFh                ; al is line number
                                        ; ebx points to the data
                                        ; ecx to pixels, edx to attributes
        mov     ecx,[dword ptr SCRLineAddress+eax*4]
        mov     edx,[dword ptr SCRAttrAddress+eax*4]

        shl     eax,6                   ; calculate physical address
        mov     ebx,eax                 ; as y * 64 + y * 256
        shl     eax,2
        add     ebx,eax
        add     ebx,[BufferPtr]         ; add buffer address
        add     ebx,4*320               ; add top of screen offset

        call    SCGetBorderPattern      ; draw the border
        mov     [ebx+0],eax             ; brute force ;-)
        mov     [ebx+4],eax
        mov     [ebx+8],eax
        mov     [ebx+12],eax
        mov     [ebx+16],eax
        mov     [ebx+20],eax
        mov     [ebx+24],eax
        mov     [ebx+28],eax
        mov     [ebx+288],eax
        mov     [ebx+292],eax
        mov     [ebx+296],eax
        mov     [ebx+300],eax
        mov     [ebx+304],eax
        mov     [ebx+308],eax
        mov     [ebx+312],eax
        mov     [ebx+316],eax

        mov     al,32                   ; do 32 characters across
        add     ebx,32                  ; add left of screen offset
_SCL0:  call    SCRenderChar            ; render char
        add     ebx,8                   ; next screen position
        inc     ecx                     ; next bit pattern
        inc     edx                     ; next attribute
        dec     al
        jnz     _SCL0
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP SCLine

; *****************************************************************************
;
;       Render a character @EBX. ^ECX is the pixel pattern, ^EDX is
;       the attribute. This is a slow renderer.
;
; *****************************************************************************

SCRenderChar Proc
        push    eax
        push    ebx
        push    ecx
        push    edx
        mov     dl,[edx]                ; dl = foreground,dh = background
        mov     dh,dl
        mov     al,dl                   ; save in al temporarily
        shr     dh,3                    ; bits 5,4,3 of attribute are BGR
        and     dl,7                    ; bits 6,2,1,0 of attribute are FGR
        test    al,64                   ; copy bit 6 ? (Bright)
        jz      SCNoBright
        or      dl,8
SCNoBright:
        test    al,128                  ; Flashing ?
        jz      SCNoFlash
        test    [FrameCount],16         ; swap fgr & bgr periodically
        jz      SCNoFlash
        xchg    dh,dl
SCNoFlash:
        mov     al,dh                   ; get bgr attribute
        and     eax,7
        mov     eax,[dword ptr SCColourTable+eax*4]
        mov     [ebx+0],eax             ; fill in the background
        mov     [ebx+4],eax             ; all in background colour
        mov     al,dl                   ; get the foreground attribute
        and     eax,15
        mov     eax,[dword ptr SCColourTable+eax*4]
        xor     edx,edx
        mov     dl,[ecx]                ; get the pixel pattern (32 bit)

                                        ; remove these three lines following
                                        ; for the slower more compact rendering
        call    [dword ptr RenderJumpTable+edx*4]
        jmp     _SCRC3
include RenGen.Inc
                                        ; *** SLOW Rendering continues... ***
        cmp     dl,255                  ; special case if all set
        je      _SCRC2
_SCRC0: test    dl,128                  ; if bit 7 set
        jz      _SCRC1
        mov     [ebx],al                ; set that attribute to foreground.
_SCRC1: inc     ebx                     ; next pixel
        add     dl,dl                   ; shift pattern left
        jnz     _SCRC0                  ; until pattern empty
_SCRC3: pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret
_SCRC2: mov     [ebx],eax               ; special case ($FF)
        mov     [ebx+4],eax
        jmp     _SCRC3

EndP SCRenderChar

.data                                   ; attribute & screen line lookup
        include ScrData.inc             ; table.
;
;       Map Speccy Colours 0-7 & Bright 0-7 onto the VGA Palette
;       Data stored as a DWord to turn four pixels on.
;
SCColourTable:
        dd      00000000h,01010101h,04040404h,05050505h ; 0-3
        dd      02020202h,03030303h,06060606h,07070707h ; 4-7
        dd      00000000h,09090909h,0c0c0c0ch,0d0d0d0dh ; Bright 0-3
        dd      0a0a0a0ah,0b0b0b0bh,0e0e0e0eh,0f0f0f0fh ; Bright 4-7
end
