; *****************************************************************************
; *****************************************************************************
;
;       File    : FileIO.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : File I/O Functions
;       Created : 29th September 1997
;       Modified: 14th October 1997
;
;       Functions :
;               FLoad           Load file into memory
;               FDirectory      Copy into the Directory Area all Snapshots
;
; *****************************************************************************
; *****************************************************************************

.386p
.model flat
include extern.inc
.code

; *****************************************************************************
;
;            Load file EDX into Memory, at memory location EBX
;                 return bytes loaded or zero if error
;
; *****************************************************************************

FLoad Proc
        push    ebx                     ; save registers
        push    ecx
        push    edx

        push    ebx                     ; save data address
        mov     al,0                    ; attempt to open file,read only
        mov     ah,03Dh                 ; filename pointer in EDX
        int     21h
        pop     edx                     ; restore address
        jc      _FX1                    ; if failed, return zero.
        push    eax                     ; save handle for close

        mov     ebx,eax                 ; read the file
        mov     ecx,SnapSize            ; try to read in max snapsize...
	mov	ah,03Fh
        int     21h

        pop     ebx                     ; Close the file - restore the handle
        push    eax                     ; eax = bytes read.
	mov	ah,03Eh
        int     21h
        pop     eax                     ; restore bytes read
        jnc     _FX2                    ; if no carry, passed ok.
_FX1:   xor     eax,eax                 ; return zero
_FX2:   pop     edx
        pop     ecx
        pop     ebx
        ret

endp FLoad

; *****************************************************************************
;
;       Do a snapshot directory. On exit EAX contains the Snapshot
;       count, EBX points to a list of filenames, 8 bytes per name
;
; *****************************************************************************

FDirectory Proc
        push    es
        mov     es,[ESSeg]
        mov     [FSnapCount],0          ; snap count = 0
        mov     ebx,offset _FDSNA       ; add *.SNA
        call    FAddSnap
        mov     ebx,offset _FDZ80       ; add *.Z80
        call    FAddSnap
        mov     ebx,offset _FDTAP       ; add *.TAP
        call    FAddSnap

_FD2:   call    FSort                   ; sort the file names
        or      eax,eax
        jnz     _FD2

        pop     es
        ret
.data                                   ; text versions of masks
ESSeg   dw      0
_FDTAP  db      '*.tap',0
_FDSNA  db      '*.sna',0
_FDZ80  db      '*.z80',0
        align   dword
.code
EndP FDirectory

; *****************************************************************************
;
;                               Add a set of snaps
;
; *****************************************************************************

FAddSnap Proc
        mov     eax,04E00h              ; Find First....
        mov     edx,ebx                 ; File mask in EDX
        xor     ecx,ecx                 ; Attributes to Zero
        int     21h                     ; Read the first one
        jc      _FAS0                   ; nothing to find.....

_FAS1:  cmp     [FSnapCount],SnapMax    ; end of snaps ?
        je      _FAS0

        push    edx                     ; save EDX
        mov     ebx,[FSnapCount]        ; EBX is the snap count number
        inc     [FSnapCount]            ; Increment the Snap Count

        mov     eax,es:[080h+01Eh]      ; Copy the file name
        mov     [dword ptr FSnapList+ebx*8],eax
        mov     eax,es:[080h+022h]
        mov     [dword ptr FSnapList+ebx*8+4],eax

        lea     ebx,ebx*8+FSnapList     ; EBX points to the new name

        push    ebx                     ; truncate after the full stop
        mov     cl,8
_FAS2:  mov     al,[ebx]
        cmp     al,'.'
        je      _FAS4
        inc     ebx
        dec     cl
        jnz     _FAS2
        jmp     _FAS5
_FAS4:  mov     [byte ptr 0+ebx],32
        inc     ebx
        dec     cl
        jnz     _FAS4
_FAS5:  pop     ebx

        pop     edx
        mov     ah,04Fh                 ; Find Next
        int     21h
        jc      _FAS1

_FAS0:  ret
EndP FAddSnap

; *****************************************************************************
;
;                            Sort the Snap Names
;
; *****************************************************************************

FSort   Proc
        xor     eax,eax
        mov     ebx,offset FSnapList
        mov     ecx,[FSnapCount]
        cmp     ecx,2
        jb      _FS0

_FS1:   push    ebx
        mov     dl,8
_FS2:   mov     dh,[ebx]
        cmp     dh,[ebx+8]
        ja      _FS3
        jb      _FS4
        inc     ebx
        dec     dl
        jnz     _FS2
        jmp     _FS4
_FS3:   inc     eax                     ; swap pair
        pop     ebx
        push    ebx
        mov     edx,[ebx]
        xchg    edx,[ebx+8]
        mov     [ebx],edx
        mov     edx,[ebx+4]
        xchg    edx,[ebx+12]
        mov     [ebx+4],edx
_FS4:   pop     ebx                     ; next pair
        add     ebx,8
        dec     ecx
        cmp     ecx,1
        ja      _FS1
_FS0:   ret

EndP FSort

.data?
FSnapCount      dd      ?
FSnapList:      dd      SnapMax dup (?)
                dd      SnapMax dup (?)
.code
end
