; *****************************************************************************
; *****************************************************************************
;
;       File    : Io.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Functions for Z80 IN and OUT commands
;       Created : 2nd October 1997
;       Modified: 10th October 1997
;
;       Functions :
;               IInPort         Read port 'AL' from port AL
;               IOutPort        Write value AH to port AL
;               IScanKeys       Scan Keys - update bits and joystick byte
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

.code 

; *****************************************************************************
;
;                  Read from port AH:AL into register AL
;
; *****************************************************************************

IInPort Proc
        test    al,020h                 ; IN 31 - the kempston port....
        jz      _IIPKempston            ; if read, must return (mainly) zero

        test    al,1                    ; read from even port only
        jnz     _IIPX

        mov     al,ah                   ; AL is the column being tested.
        mov     ah,255                  ; set the default bits

        test    al,1                    ; row 1 (Shift,Z,X,C,V)
        jnz     _IIP1
        and     ah,[BitTable+0]
_IIP1:  test    al,2                    ; row 2 (A,S,D,F,G)
        jnz     _IIP2
        and     ah,[BitTable+1]
_IIP2:  test    al,4                    ; row 3 (Q,W,E,R,T)
        jnz     _IIP3
        and     ah,[BitTable+2]
_IIP3:  test    al,8                    ; row 4 (1,2,3,4,5)
        jnz     _IIP4
        and     ah,[BitTable+3]
_IIP4:  test    al,16                   ; row 5 (0,9,8,7,6)
        jnz     _IIP5
        and     ah,[BitTable+4]
_IIP5:  test    al,32                   ; row 6 (P,O,I,Y,U)
        jnz     _IIP6
        and     ah,[BitTable+5]
_IIP6:  test    al,64                   ; row 7 (Enter,L,K,J,H)
        jnz     _IIP7
        and     ah,[BitTable+6]
_IIP7:  test    al,128                  ; row 8 (Space,SymShift,M,N,B)
        jnz     _IIP8
        and     ah,[BitTable+7]
_IIP8:
        or      ah,0E0h                 ; set upper 3 bits
        cmp     [Issue2],0              ; Issue 2 ?
        jnz     _IIP9
        and     ah,0BFh                 ; clear Bit 6 of Port for >Issue 2
_IIP9:  mov     al,ah                   ; return that value

        
        jmp     _IIP0
_IIPX:  mov     al,[byte ptr Scan]      ; unknown port, return 0xFF or Scan#
_IIP0:  ret

_IIPKempston:                           ; return the joystick port value.
;       mov     al,dbmSingleStep
;       call    BSetExecute
        mov     al,[Joystick]           ; read the joystick status
        jmp     _IIP0
EndP IInPort

BitTable db     8 dup (255)             ; Table of bit values
Joystick db     0                       ; Kempston Joystick Value

; *****************************************************************************
;
;                        Scan Keys, updating the key tables
;
; *****************************************************************************

IScanKeys Proc

        push    eax
        push    ebx
        push    ecx
        push    edx
        push    ebp

        mov     ecx,offset BitTable     ; ECX points to the data
        mov     ebp,offset ScanCodeTable; EBP points to the scan codes
        mov     dl,8                    ; DL counts the columns

ISK1:   mov     dh,255                  ; set to all 1's
        mov     [ecx],dh
        mov     dh,1                    ; the initial mask 00000001

ISK2:   mov     al,[ebp]                ; get the scan code
        and     eax,127                 ; make it 32 bits long
        test    [byte ptr KeyTable+eax],128
        jz      ISK3                    ; if not pressed, skip this bit...

        mov     al,dh                   ; al = the mask
        xor     al,255                  ; it is an AND mask
        and     [ECX],al                ; clear the appropriate bit
ISK3:
        inc     ebp                     ; look at next scan table entry
        add     dh,dh                   ; shift the mask one to the left
        cmp     dh,20h
        jne     ISK2                    ; loop back until done one column

        inc     ecx                     ; do the next line
        dec     dl                      ; do it 8 times...
        jnz     ISK1

        cmp     [Special],0             ; if specials turned off
        jz      ISK5                    ; skip this next bit...

        mov     ebx,offset Specials     ; Special Collections...
ISK4:   mov     al,[ebx]                ; key number in AL
        or      al,al
        jz      ISK5                    ; if zero end of table
        and     eax,127                 ; check if it is pressed
        test    [byte ptr KeyTable+eax],128
        jnz     ISK6                    ; if pressed set other keys...
        add     ebx,5                   ; skip to next entry in table
        jmp     ISK4
ISK6:   mov     ebx,[ebx+1]             ; get address from table
ISK7:   mov     al,[ebx]                ; get column no
        cmp     al,255                  ; 255 = end of key table
        je      ISK5
        and     eax,7                   ; mask column no
        mov     cl,[ebx+1]              ; get bit to clear
        xor     cl,255                  ; make it into a mask
        and     [BitTable+eax],cl       ; clear the bit
        add     ebx,2                   ; go to the next one
        jmp     ISK7

ISK5:   mov     [Joystick],0            ; Update the joystick byte

        test    [byte ptr KeyTable+kUp],128
        jz      _ISKJ0
        or      [Joystick],8
_ISKJ0:
        test    [byte ptr KeyTable+kDown],128
        jz      _ISKJ1
        or      [Joystick],4
_ISKJ1:
        test    [byte ptr KeyTable+kLeft],128
        jz      _ISKJ2
        or      [Joystick],2
_ISKJ2:
        test    [byte ptr KeyTable+kRight],128
        jz      _ISKJ3
        or      [Joystick],1
_ISKJ3:
        test    [byte ptr KeyTable+kAlt],128
        jz      _ISKJ4
        or      [Joystick],16
_ISKJ4:


        pop     ebp
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP IScanKeys

Specials:
        db      kBackspace              ; backspace mapped onto shift+0
        dd      kBSX
        db      kUp                     ; arrow keys
        dd      KUPX
        db      kDown
        dd      KDWX
        db      kLeft
        dd      KLFX
        db      kRight
        dd      KRGX
        db      KShiftL                 ; left shift
        dd      KRSX                    ; has to be last (2A problem)
        db      0                       ; end of specials

kBSX:   db      0,1,4,1,0FFh            ; Backspace = SHIFT + 0
kRSX:   db      0,1,0FFh                ; Right Shift = SHIFT
kLFX:   db      0,1,3,010h,0ffh         ; Arrow Keys....
kRGX:   db      0,1,4,4,0FFh
kUPX:   db      0,1,4,8,0FFh
kDWX:   db      0,1,4,16,0FFh
        align   dword
ScanCodeTable:
        db      kShiftR,kZ,kX,kC,kV
        db      kA,kS,kD,kF,kG
        db      kQ,kW,kE,kR,kT
        db      k1,k2,k3,k4,k5
        db      k0,k9,k8,k7,k6
        db      kP,kO,kI,kU,kY
        db      kEnter,kL,kK,kJ,kH
        db      kSpace,kCtrl,kM,kN,kB

; *****************************************************************************
;
;                         Write value AH to port AL
;
; *****************************************************************************

IOutPort Proc
        test    al,1                    ; write to even port only
        jnz     _IOP0
        mov     [Border],ah             ; set the border colour
        and     [Border],7              ; (and 00000111)
        and     ah,010h                 ; Isolate the sound bit
        cmp     ah,[LastSound]          ; has it toggled ?
        je      _IOP0                   ; no.... skip this
        mov     [LastSound],ah          ; save last toggle value
        call    NToggle                 ; Notify the sound routine
_IOP0:  ret
LastSound db    0                       ; last toggle value
        align   dword
EndP IOutPort

end
