; *****************************************************************************
; *****************************************************************************
;
;       File    : Keyboard.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Main Program
;       Created : 8th October 1997
;       Modified: 8th October 1997
;
;               KInitialise     Patch Int9 with new Keyboard handler
;               KTerminate      Replace Old Keyboard Handler Back
;               KTest           Display Key Scan Codes
;               KGet            Get a keystroke from list @EBX
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

.code

; ***********************************************************************
;
;           Get a key from the list at EBX, terminated in Zero
;
; ***********************************************************************

KGet Proc
        push    ebx
_KG1:   call    KGetSub                 ; wait for release
        cmp     al,255
        jnz     _KG1
_KG2:   call    KGetSub                 ; wait for depression
        cmp     al,255
        je      _KG2
        push    eax
_KG3:   call    KGetSub                 ; wait for release
        cmp     al,255
        jnz     _KG3
        pop     eax
        pop     ebx
        ret

KGetSub:push    ebx
        push    ecx
        xor     al,al
_KGS0:  xor     ecx,ecx                 ; ecx = key number
        mov     cl,[ebx]              
        or      cl,cl                   ; if end of table,exit
        jz      _KGS1
        test    [byte ptr KeyTable+ecx],128
        jnz     _KGS2                   ; if pressed, exit
        inc     ebx                     ; next entry
        inc     al                      ; next key number
        jmp     _KGS0

_KGS1:  mov     al,255                  ; return no pressed
_KGS2:  pop     ecx                     ; return
        pop     ebx
        ret

EndP KGet

; ***********************************************************************
;
;                     Patch in the Keyboard Handler
;
; ***********************************************************************

KInitialise Proc
        push    eax
        push    ebx
        push    ecx
        push    edx
        mov     ebx,offset KeyTable             ; Clear the Keyboard Table
        mov     ax,128
_KI0:   mov     [ebx],ah
        inc     ebx
        dec     al
        jnz     _KI0
        mov     ax,0204h                        ; get the old Int9 vector
        mov     bl,9                            ; and save it.
        int     31h
        mov     [Int9Selector],cx
        mov     [Int9Offset],edx
        mov     [DataSel],ds                    ; save the data selector.
        mov     ax,0205h                        ; install the new routine
        mov     bl,9
        mov     cx,cs                           ; current selector
        mov     edx,offset NewInt9
        int     031h
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP KInitialise

; ***********************************************************************
;
;                       Replace the Keyboard Handler
;
; ***********************************************************************

KTerminate Proc
        push    eax
        push    ebx
        push    ecx
        push    edx
        mov     ax,0205h                        ; restore the keyboard vector
        mov     bl,9
        mov     cx,[Int9Selector]
        mov     edx,[Int9Offset]
        int     031h
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP KTerminate

; ***********************************************************************
;
;                       Keyboard Test Program
;
; ***********************************************************************

KTest   Proc
        call    KInitialise
        xor     ebx,ebx
_KTT0:  test    [byte ptr KeyTable+1],128
        jnz     _KTT1
        test    [byte ptr KeyTable+ebx],128
        jz      _KTT2

        mov     al,bl
        shr     al,4
        call    _KTT4
        mov     al,bl
        call    _KTT4
        mov     al,32
        call    _KTT5

_KTT2:  inc     ebx
        and     ebx,127
        jmp     _KTT0

_KTT1:  call    KTerminate
        mov     ax,4C00h
        int     21h

_KTT4:  and     al,15
        cmp     al,10
        jb      _KTT6
        add     al,7
_KTT6:  add     al,48
_KTT5:  push    ebx
        mov     ah,2
        mov     dl,al
        int     21h
        pop     ebx
        ret
EndP KTest
; ***********************************************************************
;
;                            New Int 9 handler
;
; ***********************************************************************

NewInt9:push    eax ebx
        push    ds
        mov     ds,cs:[DataSel]

        in      al,60h                  ; get the scan code

        mov     bl,al                   ; bx = offset into the table
        and     ebx,07Fh

        and     al,080h                 ; isolate the on off bit
        xor     al,080h                 ; bit 7 0 if key pressed,reverse it
        mov     [byte ptr KeyTable+ebx],al ; write it into the table

        mov     al,20h                  ; reset something or other....
        out     20h,al

        pop     ds
        pop     ebx eax
        sti
        iret

.data?
KeyTable:
         db     128 dup (?)             ; keyboard table IN THE CODE SEGMENT
Int9Selector dw ?,?                     ; selector
Int9Offset   dd ?                       ; offset in selector
DataSel dd      ?                       ; Data Segment Selector

.code

end
