; *****************************************************************************
; *****************************************************************************
;
;       File    : Load_Sna.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Load a .SNA format snapshot into the system
;       Created : 2nd October 1997
;       Modified: 2nd October 1997
;
;       Functions :
;               SNALoad         Load sna format snapshot name @ EBX
;                               (returns Carry Set if Failed)
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

; *****************************************************************************
;
;       This buffer is used by all formats for loading/saving snaps
;
; *****************************************************************************
.data?
SnapBuffer:
        db   SnapSize dup (?)
.code

; *****************************************************************************
;
;                Load a .SNA format snap . Snap name in EBX
;                        Return non-zero if successful
;
; *****************************************************************************

SNALoad Proc
        mov     edx,ebx                 ; file name in EBX
        mov     ebx,offset SnapBuffer   ; EBX points to snap buffer
        call    FLoad
        or      eax,eax
        jz      _SNAL1

        mov     al,[byte ptr SnapBuffer+0]      ; I from offset 0
        mov     [byte ptr _I],al

        mov     ax,[word ptr SnapBuffer+1]      ; HL' DE' BC' AF' from 1,3,5,7
        mov     [word ptr _XHL],ax
        mov     ax,[word ptr SnapBuffer+3]
        mov     [word ptr _XDE],ax
        mov     ax,[word ptr SnapBuffer+5]
        mov     [word ptr _XBC],ax
        mov     ax,[word ptr SnapBuffer+7]
        mov     [word ptr _XAF],ax

        mov     ax,[word ptr SnapBuffer+9]      ; HL DE BC IY IX from 9-17
        mov     [word ptr _HL],ax
        mov     ax,[word ptr SnapBuffer+11]
        mov     [word ptr _DE],ax
        mov     ax,[word ptr SnapBuffer+13]
        mov     [word ptr _BC],ax
        mov     ax,[word ptr SnapBuffer+15]
        mov     [word ptr _IY],ax
        mov     ax,[word ptr SnapBuffer+17]
        mov     [word ptr _IX],ax

        mov     al,[byte ptr SnapBuffer+19]     ; IFF2 bit 2 of 19
        and     al,4
        mov     [_IFF2],al                      ; from bit 4
        mov     [_IFF1],al

        mov     al,[byte ptr SnapBuffer+20]     ; R from 20
        mov     [_R],al
        and     al,128                          ; update Bit 7 of R
        mov     [_RBit7],al

        mov     ax,[word ptr SnapBuffer+21]     ; AF,SP from 21,23
        mov     [word ptr _AF],ax
        mov     ax,[word ptr SnapBuffer+23]
        mov     [word ptr _SP],ax

        mov     al,[byte ptr SnapBuffer+25]     ; Int Mode from 25
        and     al,3
        cmp     al,3
        jne     NoResMode
        mov     al,2
NoResMode:
        mov     [_IMode],al

        mov     al,[byte ptr SnapBuffer+26]     ; Border from 26
        and     al,7
        mov     [Border],al

        mov     ebx,offset SnapBuffer+27        ; Copy 48k RAM from 27...
        mov     ecx,offset Memory+16384
        mov     edx,48*1024/4
_SNAL0: mov     eax,[ebx]
        mov     [ecx],eax
        add     ebx,4
        add     ecx,4
        dec     edx
        jnz     _SNAL0

        call    _OG2_45                         ; execute a RETN
        mov     eax,-1                          ; return non-zero
_SNAL1:
        ret
EndP SNALoad

end
