; *****************************************************************************
; *****************************************************************************
;
;       File    : Load_Tap.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Load a .TAP format file
;       Created : 12th October 1997
;       Modified: 12th October 1997
;
;       Functions :
;               TAPLoad         Load TAP format snapshot name @ EBX
;                                  (returns Carry Set if Failed)
;               TapePatch       Called on ED01 in Z80 Emulation
;               TAPPatchROM     Patch Z80 Rom
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc
.code

; *****************************************************************************
;
;                     Load a .TAP file into the emulator
;
; *****************************************************************************

TAPLoad Proc
        mov     [TAPPtr],0              ; clear the TAP Pointer
        mov     edx,ebx                 ; file name in EBX
        mov     ebx,offset SnapBuffer   ; EBX points to snap buffer
        call    FLoad
        or      eax,eax
        jz      _TAPL1
                                        ; mark end of tape buffer
        mov     [dword ptr SnapBuffer+eax],0
                                        ; if read , set the TAP Pointer
        mov     [TAPPtr],offset SnapBuffer
        call    TAPPatchROM             ; Patch ROM for TAP access
        mov     eax,-1                  ; return "loaded"

_TAPL1: ret

EndP TAPLoad

; *****************************************************************************
;
;               Patch the Spectrum ROM - calls to $056C (LOAD)
;
; *****************************************************************************

TAPPatchROM Proc
        mov     [byte ptr Memory+056Ch],0EDh    ; ED 01 (patch)
        mov     [byte ptr Memory+056Dh],001h
        ret
EndP TAPPatchROM

; *****************************************************************************
;
;          Called at a call to $0556. IX = Address,DE = Bytes to read
;
; *****************************************************************************

TapePatch Proc
        mov     ebx,[TAPPtr]                    ; EBX = Address of TAP Data
        add     ebx,3                           ; Skip Length and Type

        mov     ebp,[dword ptr _IX]             ; EBP = Destination Address
        and     ebp,0FFFFh                      ; a 16 bit value
        add     ebp,offset Memory               ; Memory+IX

        mov     cx,[word ptr _DE]               ; CX is count.

_TPT1:  mov     al,[ebx]                        ; copy CX bytes
        mov     [ebp],al
        inc     ebx
        inc     ebp
        dec     cx
        jnz     _TPT1

        mov     ebx,[TAPPtr]                    ; advance to next one
        xor     ecx,ecx
        mov     cx,[ebx]                        ; get length
        add     ebx,ecx                         ; add to pointer
        add     ebx,2                           ; add 2 (for length word)
        mov     [TAPPtr],ebx                    ; update the result

                                                ; Thanks to James McKay
                                                ; only ROM code I found
                                                ; was in German.

        mov     [word ptr _PC],05E2h            ; PC = $05E2
        call    _OG0_08                         ; EX AF,AF'
        or      [byte ptr _F],040h              ; Set the Zero Flag

        ret
 
EndP TapePatch
 
end
