; *****************************************************************************
; *****************************************************************************
;
;       File    : Load_Z80.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Load a .Z80 format snapshot into the system
;       Created : 10th October 1997
;       Modified: 18th October 1997
;
;       Functions :
;               Z80Load         Load Z80 format snapshot name @ EBX
;                               (returns Carry Set if Failed)
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc
.code

; *****************************************************************************
;
;                Load a .Z80 format snap . Snap name in EBX
;                        Return non-zero if successful
;
; *****************************************************************************

Z80Load Proc
        mov     edx,ebx                 ; file name in EDX
        mov     ebx,offset SnapBuffer   ; Load the snap file
        call    FLoad
        or      eax,eax
        jz      _Z80L1
        mov     ecx,eax                 ; save length in ECX

        mov     al,[byte ptr SnapBuffer+0]      ; +0 Accumulator
        mov     [byte ptr _A],al

        mov     al,[byte ptr SnapBuffer+1]      ; +1 Flag Register
        mov     [byte ptr _F],al

        mov     ax,[word ptr SnapBuffer+2]      ; +2 BC
        mov     [word ptr _BC],ax

        mov     ax,[word ptr SnapBuffer+4]      ; +4 HL
        mov     [word ptr _HL],ax

        mov     ax,[word ptr SnapBuffer+6]      ; +6 Program Counter (old)
        mov     [word ptr _PC],ax

        mov     ax,[word ptr SnapBuffer+8]      ; +8 Stack Pointer
        mov     [word ptr _SP],ax

        mov     al,[byte ptr SnapBuffer+10]     ; +10 Interrupt Register
        mov     [byte ptr _I],al

        mov     al,[byte ptr SnapBuffer+11]     ; +11 Refresh Register
        mov     [byte ptr _R],al

        mov     al,[byte ptr SnapBuffer+12]     ; +12 Miscellany
        cmp     al,255                          ; compatibility-255 => 1
        jne     _Z80L2
        mov     al,1
_Z80L2: mov     ah,al
        and     al,1                            ; bit 0 is bit 7 of R
        shl     al,7                            ; move it to proper place
        mov     [_RBit7],al
        shr     ah,1                            ; bits 1-3 are border colour
        and     ah,7                            ; move into position and
        mov     [Border],ah                     ; mask off

        mov     ax,[word ptr SnapBuffer+13]     ; +13 DE
        mov     [word ptr _DE],ax

        mov     ax,[word ptr SnapBuffer+15]     ; +15 BC'
        mov     [word ptr _XBC],ax

        mov     ax,[word ptr SnapBuffer+17]     ; +17 DE'
        mov     [word ptr _XDE],ax

        mov     ax,[word ptr SnapBuffer+19]     ; +19 HL'
        mov     [word ptr _XHL],ax

        mov     al,[byte ptr SnapBuffer+21]     ; +21 A'
        mov     [byte ptr _XAF+1],al

        mov     al,[byte ptr SnapBuffer+22]     ; +22 F'
        mov     [byte ptr _XAF],al

        mov     ax,[word ptr SnapBuffer+23]     ; +23 IY
        mov     [word ptr _IY],ax

        mov     ax,[word ptr SnapBuffer+25]     ; +25 IX
        mov     [word ptr _IX],ax

        mov     al,[byte ptr SnapBuffer+27]     ; +27 IFF1
        mov     [_IFF1],al

        mov     al,[byte ptr SnapBuffer+28]     ; +28 IFF2
        mov     [_IFF2],al

        mov     al,[byte ptr SnapBuffer+29]     ; +29 Int Mode / Issue 2
        mov     ah,al
        and     ah,3                            ; Int Mode = Bits 0-1
        mov     [_IMode],ah
        and     eax,4                           ; Issue 2 emulation
        mov     [Issue2],eax

        mov     ax,[word ptr SnapBuffer+6]      ; versio 1.45 or earlier
        or      ax,ax
        jnz     _Z80L8

        mov     ax,[word ptr SnapBuffer+32]     ; +32 PC (v2.01+)
        mov     [_PC],ax

        mov     al,[byte ptr SnapBuffer+34]     ; test snap type
        cmp     al,2                            ; must be 48k or 48k + IF1
        jae     _Z80L9                          ; (which isn't emulated..)

        mov     bx,[word ptr SnapBuffer+30]     ; +30 Length of Data
        and     ebx,0FFFFh                      ; make it 32 bit
        add     ebx,offset SnapBuffer+32        ; pointer to data = 32+Length

_Z80L5:                                         ; Decompression loop
        xor     eax,eax                         ; eax and ecx zeroed
        xor     ecx,ecx
        mov     cx,[ebx]                        ; ecx = snap section length
        mov     al,[ebx+2]                      ; eax = type.
        or      al,al                           ; if type is zero,finished
        jz      _Z80L6
                                                ; ebp = load address
        mov     ebp,[dword ptr Z80LoadTable+eax*4]
        add     ebx,3                           ; ebx points to start of data

        or      ebp,ebp                         ; check ebp is non-zero
        jz      _Z80L7                          ; if so,don't load
        call    Decompress                      ; decompress it
_Z80L7: add     ebx,ecx                         ; point to next block
        jmp     _Z80L5

Z80LoadTable:                                   ; load addresses
        dd      0,0,0,0
        dd      Memory+8000h,Memory+0C000h,0,0
        dd      Memory+4000h,0,0,0

_Z80L8: mov     ebx,offset SnapBuffer+30
        mov     ebp,offset Memory+4000h
        sub     ecx,30                          ; length (-header)
        call    Decompress
        jmp     _Z80L6

_Z80L6: mov     eax,-1                          ; return non-zero
        jmp     _Z80L1

_Z80L9: mov     ebx,offset CantLoad
        call    Menu
_Z80L3: xor     eax,eax                         ; failed - return 0
_Z80L1: ret

CantLoad:db     1,'Snapshot for 128k+',0
        align   dword
EndP Z80Load

; *****************************************************************************
;
;      Z80 Decompressor. EBX ^ Compressed Data,ECX = Length,EBP = Target
;
; *****************************************************************************

Decompress Proc
        push    eax
        push    ebx
        push    ecx
        push    ebp
        add     ecx,ebx                 ; ecx points to the data end

_DCP1:  cmp     ebx,ecx                 ; exit if finished decompressing
        jae     _DCP0

        mov     ax,[ebx]                ; get word at the decompress ptr
        cmp     ax,0EDEDh               ; look for compression marker
        je      _DCP2                   ; if found $EDED, decompress

        cmp     al,0EDh                 ; if found $ED copy byte and skip
        je      _DCP5

        mov     [ebp],al
        inc     ebp                     ; advance the pointers
        inc     ebx
        jmp     _DCP1                   ; decompress another

_DCP5:  add     ebx,2                   ; $ED xx just puts in $ED xx
        mov     [byte ptr ebp],ax
        add     ebp,2
        jmp     _DCP1

_DCP2:  mov     al,[ebx+2]              ; al = repeat count
        mov     ah,[ebx+3]              ; ah = repeat data
        add     ebx,4                   ; skip over repeat block
_DCP4:  mov     [ebp],ah                ; copy ah al times
        inc     ebp
        dec     al
        jnz     _DCP4
        jmp     _DCP1                   ; go and do another one.

_DCP0:  pop     ebp
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP Decompress
end
