; *****************************************************************************
; *****************************************************************************
;
;       File    : Main.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Main Program
;       Created : 12th September 1997
;       Modified: 14th October 1997
;
; *****************************************************************************
;        48k Images Tested : 134      Running Ok : 134      Faulty : 000
; *****************************************************************************
;
; To do:
;       F7 Save Snapshots
;       Kempston address decoding ?
;       Bit 3 & 5 Emulation - check Commando, Ghosts n Goblins.
;       Other Formats (TZX SLT ?)
;       Does IN(FF) return values > 191 (Scan Lines)
;
; *****************************************************************************
;
; Fixes: (for specific details see appropriate files)
;
; 1.    Emulate Bit 7 of 'R' - stopping Alien 8 from crashing - tested it
;       for some reason - would stop a snapshot of it from loading (GEN8.C)
;
; 2.    Adding Scan-line redraw fixed Arkanoid 1 (game froze) (MAIN.ASM)
;       Waiting for a given scanline
;
; 3.    16 bit Read on $FFFF failed (Heartland), copied Mem+0 to Mem+$10000
;       I was $FF reading IM2 vector from $FFFF and $0000 ! (DATA.ASM)
;
; 4.    R Bit 7 Loaded from Snap. Causing 'Marble Madness' to fail.
;       (LOAD_SNA.ASM) Tested R Bit 7 at one point.
;
; 5.    Added DD/FD CD xxxx (Call preceded with DD/FD) to fix 'Batman'
;       why does it do this ?
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

.code 

Start:  ; jmp     KTest                 ; remove ; to check scancodes
        mov     [ESSeg],es              ; save the ES
        call    CMHandler               ; Get the command line stuff
        call    KInitialise             ; Initialise new keyboard handler
        call    VSetGraphicMode         ; Switch to 300x256 mode
        call    DetectAdlib             ; Find the Adlib Card

Reset1: call    DReset                  ; Reset the CPU
        mov     [RedrawFunc],offset ScanLineFrameRedraw

        mov     eax,Ext_Z80             ; try to load  xxxx.Z80 file
        call    CMFileName              ; first
        call    Z80Load
        or      eax,eax                 ; if loaded ok run it.
        jnz     RunNow

        mov     eax,Ext_SNA             ; try to load a xxxx.SNA file
        call    CMFileName              ; second
        call    SNALoad
        or      eax,eax                 ; Loaded correctly,if so run it
        jnz     RunNow

        mov     eax,Ext_TAP             ; try to load a xxxx.TAP file
        call    CMFileName              ; third
        call    TAPLoad
        or      eax,eax                 ; Loaded correctly,if so run it
        jnz     RunBas                  ; (via BASIC)

RunBas: mov     [Special],1             ; Booting BASIC, Special Keys

RunNow: call    IScanKeys               ; Scan the keys initially
        mov     al,dbmRun               ; switch to run normal mode
        call    BSetExecute

Main1:  call    ReadPCTimer             ; Read the timer value
        push    ax                      ; and save it
        mov     eax,[FrmRate]           ; do FrmRate frames
Main11: push    eax
        call    OneFrame
        pop     eax
        dec     eax
        jnz     Main11
        pop     bx
        cmp     [FrmRate],9             ; Frame rate 9 ignores timer
        je      Main13
Main12: call    ReadPCTimer             ; wait for clock to change
        cmp     ax,bx
        je      Main12

Main13: test    [byte ptr KeyTable+kF2],128
        jnz     Configure               ; F2 is configure
        test    [byte ptr KeyTable+kF1],128
        jnz     Reset1                  ; F1 is Reset
        test    [byte ptr KeyTable+kF5],128
        jnz     LoadFromMenu            ; F1 is load from menu
        test    [byte ptr KeyTable+kEsc],128
        jz      Main1                   ; Loop back if ESC not pressed
        jmp     GoDOS                   ; return to MSDOS

BError: call    BDebug                  ; Come here on illegal opcode
        jmp     Main1

Configure:
        call    CNMenu
        jmp     Main1
     
LoadFromMenu:
        call    MenuLoad                ; Get the file name
        or      eax,eax                 ; was it 'cancel'
        jz      Main1                   ; if so, do nothing
        mov     [FileName],eax          ; if not, its the new filename
        jmp     Reset1                  ; Reset & Load

GoDOS:  call    NSoundOff
        call    VSetTextMode
        call    KTerminate
        mov     ax,04c00h               ; return to MSDOS
        int     21h

SFRMsg  db      'Frame Rate is '        ; text messages
SFRDat  db      0,' per PC tick.',0
SFSRMsg db      'Display 1 in '
SFSRDat db      0,' frames.',0
FSRMsg  db      'Single Refresh Redraw',0
FSLRMsg db      'Scanline Redraw',0
MsOn    db      'Sound on',0
MsOff   db      'Sound off',0

        align   dword

; *****************************************************************************
;
;                            Single-Redraw Frame
;
; *****************************************************************************

FrameSingleRedraw Proc
        mov     eax,[Cycles]            ; work out end-of-screen cycle value
        mov     [Scan],-1
        add     eax,312*224
        mov     [ECycles],eax
_FSR0:  call    [ExecPtr]               ; do one command
        mov     eax,[Cycles]            ; until sufficient cycles done.
        cmp     eax,[ECycles]
        jb      _FSR0
        cmp     [FrameDraw],0           ; drawing the frame ?
        jz      _FSR1
        call    SCRebuild               ; Redraw the screen
_FSR1:  ret
ECycles dd      0
EndP FrameSingleRedraw

; *****************************************************************************
;
;                   Handle the "Frame Draw" flag setting
;
; *****************************************************************************

FrameSkipProcess Proc
        mov     [FrameDraw],0           ; clear the frame draw flag
        dec     [FSCounter]             ; decrement the frameskip counter
        jg      _FSP0                   ; if >= 0 not drawing frame now
        mov     [FrameDraw],1           ; set the frame drawing flag
        mov     eax,[FrmSkip]           ; Reset the frame skip counter
        mov     [FSCounter],eax
_FSP0:  ret
FrameDraw dd    0                       ; When non-zero drawing frame
FSCounter dd    0                       ; Counter for frame skipping
EndP FrameSkipProcess

; *****************************************************************************
;
;                   Check Interrupt & Execute if Enabled
;
; *****************************************************************************

CheckInterrupt Proc
        mov     al,[_Iff1]              ; this is an IRQ - check to see
        or      al,al                   ; if it is enabled
        jz      _CI0

        mov     [_IFF2],0               ; IFF2 = 0
        mov     [_IFF1],0               ; IFF1 = 0

        mov     ebx,[dword ptr _PC]     ; if next command is $76 (Halt)
        mov     al,[byte ptr Memory+ebx]
        cmp     al,076h
        jne     _CI1
        inc     [word ptr _PC]          ; bump PC (wake it up)

_CI1:   cmp     [_IMode],2              ; IM2 ?
        je      _CI2
        call    _OG0_FF                 ; IM1 & 0 both do RST38h
        jmp     _CI0

_CI2:   call    _OG2_FF                 ; IM2 does CALL (I)FFh

_CI0:   ret

EndP CheckInterrupt

; *****************************************************************************
;
;                     Get a keystroke from 1-9 in AL
;
; *****************************************************************************

Get1To9 Proc
        call    NSoundOff
        mov     ebx,offset _Get1To9
        call    KGet
        inc     al
        ret
_Get1To9 db     k1,k2,k3,k4,k5,k6,k7,k8,k9,0
EndP Get1To9

; *****************************************************************************
;
;                              Do a single frame
;
; *****************************************************************************

OneFrame Proc
        mov     [Cycles],0              ; Reset the Cycle Count
        test    [byte ptr KeyTable+kF10],128
        jz      _NoDebug                ; test for F10 (enter debugger)
        call    BDebug
_NoDebug:
        call    CheckInterrupt          ; Do an IRQ test at VBLANK
        call    NNewFrame               ; Start of Frame sound call
        call    [RedrawFunc]            ; Do one frame
        call    NEndFrame               ; End of Frame sound call

        call    IScanKeys               ; Update the key patterns
        call    FrameSkipProcess        ; check for FrameSkip
        inc     [FrameCount]            ; next frame
        ret
RedrawFunc dd   0
EndP OneFrame

; ***********************************************************************
;
;                      Read the PC's 18.2 Hz Timer
;
; ***********************************************************************

ReadPCTimer Proc
        push    ebx ecx edx
        mov     ah,0                    ; look at the clock tick
        int     01Ah
        mov     ax,dx                   ; put the tick in AX
        pop     edx ecx ebx
        ret
EndP ReadPCTimer

; *****************************************************************************
;
;                          Scanline Redraw Frame
;
; *****************************************************************************

ScanlineFrameRedraw Proc
        mov     eax,[Cycles]            ; work out end-of-screen cycle value
        mov     [Target],eax
        mov     [Scan],-1               ; in border

                                        ; draw the top border
        add     [Target],64*224         ; TSStart Cycles
_SFR0:  call    [ExecPtr]               ; typical loop thingy
        mov     eax,[Cycles]
        cmp     eax,[Target]
        jb      _SFR0
        cmp     [FrameDraw],0           ; drawing frame ?
        jz      _SFR4
        call    SCTopBorder
_SFR4:

        mov     [Scan],0                ; Do visual bit of the screen
_SFR1:  mov     eax,224                 ; do TSLine Cycles per ScanLine
        add     [Target],eax
_SFR2:  call    [ExecPtr]
        mov     eax,[Cycles]
        cmp     eax,[Target]
        jb      _SFR2
        cmp     [FrameDraw],0           ; drawing frame ?
        jz      _SFR3
        mov     ax,[Scan]               ; draw scanline for this line
        call    SCLine
_SFR3:  inc     [Scan]
        mov     ax,[Scan]
        cmp     ax,192                  ; do it SLCount times
        jne     _SFR1

                                        ; draw the bottom border
        add     [Target],56*224         ; TSStart Cycles
        mov     [Scan],-1
        mov     eax,[Cycles]
        cmp     eax,[Target]
        jae     _SFR7

_SFR5:  call    [ExecPtr]               ; typical loop thingy
        mov     eax,[Cycles]
        cmp     eax,[Target]
        jb      _SFR5
_SFR7:  cmp     [FrameDraw],0           ; drawing frame ?
        jz      _SFR6
        call    SCBottomBorder
_SFR6:
        ret

Target  dd      0

EndP ScanLineFrameRedraw


end Start
