; *****************************************************************************
; *****************************************************************************
;
;       File    : Menu.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Menu Functions
;       Created : 13th October 1997
;       Modified: 14th October 1997
;
;       Functions :
;               Menu            Display & Select from Menu
;               MenuLoad        Display & Select from available snaps
;
; *****************************************************************************
; *****************************************************************************

.386p
.model flat
include extern.inc
.code

; *****************************************************************************
;
;       Popup style menu. EBX points to a menu, first byte is number of
;       items, followed by ASCIIZ strings. Returns Menu Item Number or 0
;       (for ESCAPE) in EAX.
;
; *****************************************************************************


Menu Proc
        push    ebx
        push    ebp
        call    NSoundOff
        mov     ebp,ebx                 ; menu pointer goes into EBP
        mov     al,24                   ; al = (24 - height)/2
        sub     al,[ebp]
        shr     al,1
        mov     [_MYPos],al             ; this is the vertical position
        mov     [_MSel],1               ; first entry selected
_MNU1:  mov     al,0                    ; Main Loop
_MNU2:  call    _MNURedraw              ; Redraw one entry
        inc     al
        cmp     al,[ebp]                ; reached the end ?
        jle     _MNU2
        call    _MNURedraw              ; one before,one after->border

        mov     ebx,offset _MNUKey      ; wait for a key press
        call    KGet
        cmp     al,4                    ; key 4 = Escape
        je      _MNU3
        cmp     al,2                    ; key 2 & key 3 (Enter,Space)
        je      _MNU4                   ; both select current
        cmp     al,3
        je      _MNU4
        cmp     al,0
        je      _MNU5                   ; 1 is up
        cmp     al,1
        je      _MNU6                   ; 2 is down
        cmp     al,5
        je      _MNU7                   ; 5 is Home
        cmp     al,6
        je      _MNU8                   ; 6 is End
        jmp     _MNU1                   ; loop back

_MNU5:  dec     [_MSel]                 ; move up
        jnz     _MNU1
_MNU8:  mov     al,[ebp]
        mov     [_MSel],al
        jmp     _MNU1

_MNU6:  inc     [_MSel]                 ; move down
        mov     al,[_MSel]
        cmp     al,[ebp]
        jle     _MNU1
_MNU7:  mov     [_MSel],1
        jmp     _MNU1

_MNU4:  xor     eax,eax                 ; return current selection
        mov     al,[_MSel]
        jmp     _MNU0
_MNU3:  xor     eax,eax                 ; return failed.
_MNU0:  pop     ebp
        pop     ebx
        ret
_MNUKey:db      kUp,kDown,kEnter,kSpace,kEsc,kHome,kEnd,0

_MNURedraw:
        push    eax
        push    ebx
        push    ecx
        mov     ah,al                   ; save entry number in AH
        mov     ebx,ebp                 ; EBX = Menu Data
        call    MenuGet                 ; get the menu item into EBX
        add     al,_MYPos               ; AL = Y Position
        add     al,VCSetY               ; make it a video command
        call    VPrint
        mov     al,VCSetX+33
        call    VPrint
        mov     al,VCSetBack+1          ; Set video colours
        call    VPrint
        mov     al,VCSetColour+15
        call    VPrint
        mov     al,' '
        call    VPrint
        mov     al,VCSetX+6
        call    VPrint
        mov     al,' '
        call    VPrint
        cmp     ah,[_MSel]              ; if selected,different colours
        jne     _MNUR3
        mov     al,VCSetBack+14
        call    VPrint
        mov     al,VCSetColour+4
        call    VPrint
_MNUR3: mov     cl,26                   ; draw solid bar bit
_MNUR0: mov     al,32
        call    VPrint
        dec     cl
        jnz     _MNUR0

        mov     al,VCSetX+9             ; reposition cursor
        call    VPrint
        or      ebx,ebx
        jz      _MNUR2
        call    VPrintString            ; Display menu item name
_MNUR2: pop     ecx
        pop     ebx
        pop     eax
        ret

.data
_MSel   db      1                       ; Currently selected entry
_MYPos  db      4                       ; Y Position of Menu
        align dword
.code
EndP Menu

; *****************************************************************************
;
;       Filename Menu. Returns Zero in EAX or the address of the file
;
; *****************************************************************************

MenuLoad Proc
        call    FDirectory              ; load the snaps
        cmp     [FSnapCount],0          ; any snaps ?
        jz      _ML0                    ; if not,nothing to load
        mov     [MCurrent],0            ; current = 0

_ML3:   mov     ebx,offset _Base0+6     ; point to first
        mov     ebp,13                  ; number to copy

_ML1:   mov     eax,[MCurrent]          ; number of entry
        cmp     eax,[FSnapCount]        ; wrap around ?
        jb      _ML2
        xor     eax,eax
        mov     [MCurrent],eax
_ML2:   lea     ecx,eax*8+FSnapList     ; address of name
        mov     edx,[ecx]               ; copy it in
        mov     [ebx],edx
        mov     edx,[ecx+4]
        mov     [ebx+4],edx
        add     ebx,_Base1-_Base0
        inc     [MCurrent]
        dec     ebp
        jnz     _ML1

        mov     ebx,offset LoadMenu
        call    Menu                    ; get the menu data
        or      eax,eax                 ; was it zero (e.g. cancel)
        jz      _ML0                    ; we must return zero
        cmp     eax,1                   ; if it was 1
        je      _ML3                    ; reload the file list

        push    ebx
        call    MenuGet                 ; get the name into EBX
        mov     eax,[ebx+6]             ; copy the text name into buffer
        mov     [MSelect],eax
        mov     eax,[ebx+10]
        mov     [MSelect+4],eax
        mov     eax,offset MSelect      ; return it
        pop     ebx
_ML0:   ret

MCurrent dd     0
MSelect dd      0,0,0

LoadMenu:
        db      14
        db      'More Files ...',0
_Base0: db      'Load "xxxxxxxx"',0
_Base1: db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0

        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0

        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0

        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        db      'Load "xxxxxxxx"',0
        align dword
EndP MenuLoad

; *****************************************************************************
;
;       AL is menu item number. EBX points to menu. Return text address
;       of appropriate menu item in EBX
;
; *****************************************************************************

MenuGet Proc
        push    eax
        cmp     al,0                    ; was it zero ?
        jz      _MG1                    ; return zero in EBX
        cmp     al,[ebx]                ; out of range ?
        ja      _MG1                    ; return zero in EBX
        inc     ebx                     ; skip menu count
_MG2:   dec     al                      ; if finished, exit
        jz      _MG0
_MG3:   mov     ah,[ebx]                ; skip over a string
        inc     ebx
        or      ah,ah
        jnz     _MG3
        jmp     _MG2                    ; try again...
_MG1:   xor     ebx,ebx
_MG0:   pop     eax
        ret
EndP MenuGet
end
