; *****************************************************************************
; *****************************************************************************
;
;       File    : Video.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Interface to the Video System
;       Created : 11th September 1997
;       Modified: 10th October 1997
;
;       Functions :
;               VSetGraphicMode         Set Graphic Mode
;               VSetTextMode            Set Text Mode
;               VPrint                  Print Character AL
;               VPrintByte              Print Byte AL
;               VPrintWord              Print Word AX
;               VPrintString            Print String @EBX (ASCIIZ)
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

.data
; *****************************************************************************
;
;                        Private Data for Video Handler
;
; *****************************************************************************
.data
        align   dword

VideoRAM dd     0                       ; offset to VGA RAM Space
LineSize dd     0                       ; Line Size (in bytes)

ForeCol  dd     0                       ; Foreground Colour
BackCol  dd     0                       ; Background Colour
XCursor  dd     0                       ; X Cursor Position
YCursor  dd     0                       ; Y Cursor Position

.code 

; *****************************************************************************
;
;            Switch to VGA Graphics Mode & Initialise Everything
;
; *****************************************************************************

VSetGraphicMode Proc
        push    eax
        push    ebx
        push    ecx
        mov     ah,0                    ; switch to 320x200 VGA mode
        mov     al,013h
        int     010h
        push    ds                      ; bx = descriptor for data seg
        pop     ebx                     ; (its a 32 bit push !)
        mov     ax,6                    ; DPMI function 6 gets the
        int     31h                     ; base address for a segment.
        shl     ecx,16                  ; into CX:DX - copy this into
        mov     cx,dx                   ; ECX
        mov     ebx,0A0000h             ; if we subtract it from A0000
        sub     ebx,ecx                 ; we get, in EBX, the offset
        mov     [VideoRAM],ebx          ; save the video ram offset

        mov     [LineSize],320          ; reset video values
        mov     [ForeCol],15
        mov     [BackCol],0
        mov     [XCursor],0
        mov     [YCursor],0

        call    VClearAll               ; clear the screen

        mov     eax,[VideoRAM]          ; intialise the speccy display
        call    SCInitialise

        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP VSetGraphicMode

; *****************************************************************************
;
;                  Return to Dos 'Text' Mode (80x25 Colour)
;
; *****************************************************************************

VSetTextMode Proc
        mov     ah,0                    ; switch to text VGA mode
        mov     al,3h
        int     010h
        ret
EndP VSetTextMode

; *****************************************************************************
;
;                              Print a String
;
; *****************************************************************************

VPrintString Proc
        push    eax
        push    ebx
VPSLoop:mov     al,[ebx]
        or      al,al
        jz      VPSExit
        call    VPrint
        inc     ebx
        jmp     VPSLoop
VPSExit:pop     ebx
        pop     eax
        ret
EndP VPrintString

; *****************************************************************************
;
;                             Print Bytes & Words
;
; *****************************************************************************

VPrintWord Proc
        push    eax
        mov     al,ah
        call    VPrintByte
        pop     eax
        call    VPrintByte
        ret
EndP VPrintWord

VPrintByte Proc
        push    eax
        shr     al,4
        call    VPrintNibl
        pop     eax
VPrintNibl:
        push    eax
        and     eax,15
        cmp     al,10
        jb      VPN1
        add     al,7
VPN1:   add     al,48
        call    VPrint
        pop     eax
        ret
EndP VPrintByte

; *****************************************************************************
;
;                              Print a Character
;
; *****************************************************************************

VPrint Proc
        push    eax
        push    ebx
        push    ecx
        push    ebp
        mov     cl,al                   ; Save the character
        cmp     cl,128                  ; check if control
        jae     VPControl
        cmp     cl,32
        jb      VPControl
        mov     eax,[YCursor]           ; EAX = Y Cursor * LineSize
        imul    [Word Ptr LineSize]
        shl     eax,3                   ; char is 8 pixels high
        add     eax,[VideoRAM]          ; point it into video space
        mov     ebp,[XCursor]           ; add X Cursor * 8
        shl     ebp,3
        add     ebp,eax                 ; got the actual pointer now

        mov     al,[Byte Ptr BackCol]   ; extend byte into EAX
        mov     ah,al
        mov     bx,ax
        shl     eax,16
        mov     ax,bx
        push    ebp
        mov     bl,8                    ; write 8 lines of background
VPClear:mov     [ebp],eax
        mov     [ebp+4],eax
        add     ebp,[LineSize]
        dec     bl
        jnz     VPClear
        pop     ebp

        xor     ebx,ebx                 ; scan the text table
        cmp     cl,'a'                  ; check for lower case
        jb      VPScan1
        cmp     cl,'z'
        ja      VPScan1
        sub     cl,32
VPScan1:cmp     [Byte Ptr TextChars+ebx],cl
        je      VPScan2                 ; if found matching , completed.
        inc     ebx
        mov     al,[Byte Ptr TextChars+ebx]
        or      al,al                   ; search till end of table
        jne     VPScan1
        xor     ebx,ebx                 ; print a space.
VPScan2:shl     ebx,3                   ; point to appropriate character
        add     ebx,offset TextTable

        mov     cl,8                    ; ebx=Char Data,ebp = Video Addr
        mov     ch,[Byte Ptr ForeCol]   ; ch = foreground col,cl = counter
VPDraw1:mov     al,[ebx]                ; get byte to scan
        push    ebp                     ; save pointer to screen
VPDraw2:test    al,128                  ; if bit set
        jz      VPDraw3
        mov     [ebp],ch                ; draw on screen
VPDraw3:inc     ebp                     ; next pixel
        add     al,al
        jnz     VPDraw2                 ; do the whole line
        pop     ebp                     ; restore start of line
        add     ebp,[LineSize]          ; next line down
        inc     ebx                     ; next bit of character data
        dec     cl                      ; do 8 lines
        jnz     VPDraw1

        inc     [XCursor]               ; advance the pointer
        cmp     [XCursor],40
        jb      VPAdv1
VPCRLF: mov     [XCursor],0             ; new line
        inc     [YCursor]
        cmp     [YCursor],25
        jb      VPAdv1
        mov     [YCursor],0
VPAdv1: jmp     VPExit

VPCls:  call    VClearAll               ; general clear screen
        mov     ebx,[VideoRAM]          ; clear the screen
        mov     al,[Byte Ptr BackCol]
        mov     ch,25*8
VPCls1: mov     cl,320/2
        push    ebx
VPCls2: mov     [ebx],al
        inc     ebx
        mov     [ebx],al
        inc     ebx
        dec     cl
        jnz     VPCls2
        pop     ebx
        add     ebx,[LineSize]
        dec     ch
        jnz     VPCls1
        mov     [XCursor],0
        mov     [YCursor],0
        jmp     VPExit


VPControl:                              ; go here if control character
        cmp     cl,13                   ; 13 is Carriage Return
        je      VPCRLF
        cmp     cl,VCClear              ; 12 is Clear Screen
        je      VPCls
        cmp     cl,128                  ; all others < 128 ignored.
        jb      VPExit
        cmp     cl,VCSetColour+16       ; 128-143 set foreground
        jb      VPSetFgr
        cmp     cl,VCSetBack+16         ; 144-159 set background
        jb      VPSetBgr
        cmp     cl,VCSetX+40            ; 160-200 set X
        jb      VPSetX
        cmp     cl,VCSetY+24            ; 200-223 set Y
        jb      VPSetY
        jmp     VPExit

VPSetBgr:and    ecx,15                  ; set background/foreground
        mov     [BackCol],ecx
        jmp     VPExit
VPSetFgr:and    ecx,15
        mov     [ForeCol],ecx
        jmp     VPExit

VPSetX: sub     ecx,VCSetX              ; set position (x & y)
        and     ecx,63 
        mov     [XCursor],ecx
        jmp     VPExit
VPSetY: sub     ecx,VCSetY
        and     ecx,31
        mov     [YCursor],ecx
        jmp     VPExit

VPExit: pop     ebp
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP VPrint

; *****************************************************************************
;
;               Clear whole screen function, Mode Specific
;
; *****************************************************************************

VClearAll Proc
        push    eax
        push    ebx
        push    ecx
        push    edx
        mov     ebx,[VideoRAM]
        mov     ecx,320*200
        xor     eax,eax
VCLoop1:mov     [ebx],al
        inc     ebx
        dec     ecx
        jnz     VCLoop1
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret
EndP VClearAll

TextChars:      db      ' 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ().",+->$!:[]{''@_',0

TextTable:      db      00000000b       ; character 0 (space)
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b

                db      01111100b       ; character 1 (0)
                db      11000110b
                db      11001110b
                db      11010110b
                db      11100110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      00011000b       ; character 2 (1)
                db      00111000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      01111110b
                db      00000000b

                db      01111100b       ; character 3 (2)
                db      11000110b
                db      00000110b
                db      01111100b
                db      11000000b
                db      11000000b
                db      11111110b
                db      00000000b

                db      01111100b       ; character 4 (3)
                db      11000110b
                db      00000110b
                db      01111100b
                db      00000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      00001100b       ; character 5 (4)
                db      00011100b
                db      00111100b
                db      01101100b
                db      11111110b
                db      00001100b
                db      00001100b
                db      00000000b

                db      11111110b       ; character 6 (5)
                db      11000010b
                db      11000000b
                db      11111100b
                db      00000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      01111100b       ; character 7 (6)
                db      11000110b
                db      11000000b
                db      11111100b
                db      11000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      11111110b       ; character 8 (7)
                db      11000110b
                db      00001100b
                db      00011000b
                db      00110000b
                db      01100000b
                db      11000000b
                db      00000000b

                db      01111100b       ; character 9 (8)
                db      11000110b
                db      11000110b
                db      01111100b
                db      11000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      01111100b       ; character 10 (9)
                db      11000110b
                db      11000110b
                db      01111110b
                db      00000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      00111000b       ; character 11 (A)
                db      01101100b
                db      11000110b
                db      11111110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      00000000b

                db      11111100b       ; character 12 (B)
                db      11000110b
                db      11000110b
                db      11111100b
                db      11000110b
                db      11000110b
                db      11111100b
                db      00000000b

                db      01111100b       ; character 13 (C)
                db      11000110b
                db      11000000b
                db      11000000b
                db      11000000b
                db      11000110b
                db      01111100b
                db      00000000b

                db      11111000b       ; character 14 (D)
                db      11001100b
                db      11000110b
                db      11000110b
                db      11000110b
                db      11001100b
                db      11111000b
                db      00000000b

                db      11111110b       ; character 15 (E)
                db      11000000b
                db      11000000b
                db      11111110b
                db      11000000b
                db      11000000b
                db      11111110b
                db      00000000b

                db      11111110b       ; character 16 (F)
                db      11000000b
                db      11000000b
                db      11111110b
                db      11000000b
                db      11000000b
                db      11000000b
                db      00000000b

                db      01111100b       ; character 17 (G)
                db      11000110b
                db      11000000b
                db      11011110b
                db      11000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      11000110b       ; character 18 (H)
                db      11000110b
                db      11000110b
                db      11111110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      00000000b

                db      01111110b       ; character 19 (I)
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      01111110b
                db      00000000b

                db      11111110b       ; character 20 (J)
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      11011000b
                db      01110000b
                db      00000000b

                db      11000110b       ; character 21 (K)
                db      11001100b
                db      11011000b
                db      11110000b
                db      11011000b
                db      11001100b
                db      11000110b
                db      00000000b

                db      11000000b       ; character 22 (L)
                db      11000000b
                db      11000000b
                db      11000000b
                db      11000000b
                db      11000000b
                db      11111110b
                db      00000000b

                db      11000110b       ; character 23 (M)
                db      11000110b
                db      11101110b
                db      11010110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      00000000b

                db      11000110b       ; character 24 (N)
                db      11000110b
                db      11100110b
                db      11010110b
                db      11001110b
                db      11000110b
                db      11000110b
                db      00000000b

                db      01111100b       ; character 25 (O)
                db      11000110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      11111100b       ; character 26 (P)
                db      11000110b
                db      11000110b
                db      11111100b
                db      11000000b
                db      11000000b
                db      11000000b
                db      00000000b

                db      01111100b       ; character 27 (Q)
                db      11000110b
                db      11000110b
                db      11000110b
                db      11001110b
                db      11000100b
                db      01111010b
                db      00000000b

                db      11111100b       ; character 28 (R)
                db      11000110b
                db      11000110b
                db      11111100b
                db      11011000b
                db      11001100b
                db      11000110b
                db      00000000b

                db      01111100b       ; character 29 (S)
                db      11000110b
                db      11000000b
                db      01111100b
                db      00000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      01111110b       ; character 30 (T)
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00000000b

                db      11000110b       ; character 31 (U)
                db      11000110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      11000110b
                db      01111100b
                db      00000000b

                db      11000110b       ; character 32 (V)
                db      11000110b
                db      11000110b
                db      11000110b
                db      01101100b
                db      00111000b
                db      00010000b
                db      00000000b

                db      11000110b       ; character 33 (W)
                db      11000110b
                db      11000110b
                db      11010110b
                db      11101110b
                db      11000110b
                db      11000110b
                db      00000000b

                db      11000110b       ; character 34 (X)
                db      01101100b
                db      00111000b
                db      00111000b
                db      00111000b
                db      01101100b
                db      11000110b
                db      00000000b

                db      11000011b       ; character 35 (Y)
                db      01100110b
                db      00111100b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00000000b

                db      11111110b       ; character 36 (Z)
                db      00000110b
                db      00001100b
                db      00011000b
                db      00110000b
                db      01100000b
                db      11111110b
                db      00000000b

                db      00011000b       ; character 37 (()
                db      00110000b
                db      01100000b
                db      01100000b
                db      01100000b
                db      00110000b
                db      00011000b
                db      00000000b

                db      00110000b       ; character 38 ())
                db      00011000b
                db      00001100b
                db      00001100b
                db      00001100b
                db      00011000b
                db      00110000b
                db      00000000b

                db      00000000b       ; character 39 (.)
                db      00000000b
                db      00000000b
                db      00000000b
                db      00111000b
                db      00111000b
                db      00000000b
                db      00000000b

                db      01101100b       ; character 40 (")
                db      01101100b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b

                db      00000000b       ; character 41 (,)
                db      00000000b
                db      00000000b
                db      00000000b
                db      00111000b
                db      00111000b
                db      01110000b
                db      00000000b

                db      00011000b       ; character 42 (+)
                db      00011000b
                db      00011000b
                db      01111110b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00000000b

                db      00000000b       ; character 43 (-)
                db      00000000b
                db      00000000b
                db      11111110b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b

                db      11110000b       ; character 44 (>)
                db      11111000b
                db      01111100b
                db      00111110b
                db      01111100b
                db      11111000b
                db      11110000b
                db      00000000b

                db      00011000b       ; character 45 ($)
                db      01111100b
                db      11000000b
                db      01111110b
                db      00000011b
                db      01111110b
                db      00011000b
                db      00000000b

                db      00011000b       ; character 46 (!)
                db      00111100b
                db      00111100b
                db      00011000b
                db      00000000b
                db      00011000b
                db      00011000b
                db      00000000b

                db      00000000b       ; character 47 (:)
                db      00000000b
                db      00011000b
                db      00011000b
                db      00000000b
                db      00011000b
                db      00011000b
                db      00000000b

                db      00000000b       ; character 48 ([)
                db      00000000b
                db      00000000b
                db      11111111b
                db      11111111b
                db      00000000b
                db      00000000b
                db      00000000b

                db      00011000b       ; character 49 (])
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b
                db      00011000b

                db      00011000b       ; character 50 ({)
                db      00011000b
                db      00011000b
                db      11111111b
                db      11111111b
                db      00000000b
                db      00000000b
                db      00000000b

                db      00011000b       ; character 51 (')
                db      00011000b
                db      00110000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b

                db      01111100b       ; character 52 (@)
                db      11000110b
                db      11000110b
                db      11011110b
                db      11010110b
                db      11010110b
                db      00011110b
                db      00000000b

                db      00000000b       ; character 53 (_)
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      00000000b
                db      11111110b
                db      00000000b

End
