/********************************************************************/
/********************************************************************/
/*																	*/
/*					Atomic (Low Level) Functions					*/
/*																	*/
/*	Handles, fetch, read , write , in & out of 8 (and 16...) bit	*/
/*	quantities														*/
/*																	*/
/********************************************************************/
/********************************************************************/

#include "z80.h"

/********************************************************************/
/*																	*/
/*			  Fetch a single byte to an 8-bit register				*/
/*																	*/
/********************************************************************/

void Fetch(char *Reg86)
{
ASSERT(InString(Reg86,"al:ah:cl:ch:dl:dh:"));
printf("    mov ebx,[dword ptr _PC]         ; fetch to %s\n",Reg86);
printf("    inc [Word Ptr _PC]\n");
Read(Reg86);							/* Read the byte there */
}

/********************************************************************/
/*																	*/
/*  		  Fetch a 16 bit word to a given register				*/
/*																	*/
/********************************************************************/

void FetchWord(char *Reg86)
{
ASSERT(InString(Reg86,"ax:bx:cx:dx:"));
printf("    mov ebx,[dword ptr _PC]         ; fetch to %s\n",Reg86);
printf("    add [Word Ptr _PC],2\n");
ReadWord(Reg86);						/* Read the word there */
}

/********************************************************************/
/*																	*/
/*			  Read a single byte at address EBX (32 bit)			*/
/*																	*/
/********************************************************************/

void Read(char *Reg86)
{
ASSERT(InString(Reg86,"al:ah:cl:ch:dl:dh:"));
printf("    mov %s,[Byte Ptr Memory+ebx]    ; read byte [ebx] to %s\n",Reg86,Reg86);
}

/********************************************************************/
/*																	*/
/*				  Read a word at address EBX (32 bit)				*/
/*																	*/
/********************************************************************/

void ReadWord(char *Reg86)
{
ASSERT(InString(Reg86,"ax:bx:cx:dx:si:"));
printf("    mov %s,[Word Ptr Memory+ebx]    ; read word [ebx] to %s\n",Reg86,Reg86);
}

/********************************************************************/
/*																	*/
/*					 Write a byte at address EBX					*/
/*																	*/
/********************************************************************/

void Write(char *Reg86)
{
int n = Alloc(1);                       /* Don't write to 0000-3FFF ROM */
ASSERT(InString(Reg86,"al:ah:cl:ch:dl:dh:"));
printf("    cmp bh,040h                     ; write %s to [ebx]\n",Reg86);
printf("    jb  short %s\n",SysLabel(n));
printf("    mov [Byte Ptr Memory+ebx],%s\n",Reg86);
printf("%s:\n",SysLabel(n));
}

/********************************************************************/
/*																	*/
/*					 Write a word at address EBX					*/
/*																	*/
/* 	 Note : possible problem with EBX = $FFFF... add extra space	*/
/*		    to memory buffer to compensate							*/
/*																	*/
/********************************************************************/

void WriteWord(char *Reg86)
{
int n = Alloc(1);
char Reg[3];
ASSERT(InString(Reg86,"ax:cx:dx:"));	/* Don't write to 0000-3FFF ROM */
printf("    cmp bh,040h                     ; write %s to [ebx]\n",Reg86);
printf("    jb  short %s\n",SysLabel(n));
printf("    mov [Word Ptr Memory+ebx],%s\n",Reg86);
printf("%s:\n",SysLabel(n));
}
