/********************************************************************/
/********************************************************************/
/*																	*/
/*					Code Generation File 1							*/
/*																	*/
/********************************************************************/
/********************************************************************/

#include "z80.h"

static void GenerateIncDecTable(void);

static char *Reg[] = { "b","c","d","e","h","l","(hl)","a" };
static char *ALU[] = { "add","adc","sub","sbc","and","xor","or","cp" };

/********************************************************************/
/*																	*/
/*						Z80 Code Generation							*/
/*																	*/
/*	Generates all the 8-bit ld <reg>,<reg> instructions				*/
/*	Generates [ALU] a,<constant>	(e.g. add a,42)					*/
/*	Generates [ALU] a,<Register>	(e.g. add a,c)					*/
/*	Generates inc reg8,dec reg8,ld reg8,<const>						*/
/*																	*/
/********************************************************************/

void Gen1(void)
{
int i,r1,r2;

/* Generate Commands 40-7F (not 76) LD reg,reg */

for (r1 = 0;r1 < 8;r1++)				/* Work through registers */
	{
	for (r2 = 0;r2 < 8;r2++)        	/* r1 is target,r2 is source */
		{
		if (r1 != 6 || r2 != 6)			/* Stops ld (hl),(hl) */
			{
			Define(G_STD,				/* Do the first bit */
				   r1 * 8 + r2 + 0x40,
				   (r1 == 6 || r2 == 6) ? 7 : 4,
				   "ld %s,%s",Reg[r1],Reg[r2]);
			if (r1 != r2)				/* If ld same do nothing ! */
				{
				if (r2 == 6)			/* Get the first reg into AL */
					{
					printf("    mov ebx,[dword ptr _HL]\n");
					Read("al");
					}
				else
					printf("    mov al,[byte ptr _%s]\n",Reg[r2]);

				if (r1 == 6)			/* Write al to the second reg */
					{
					printf("    mov ebx,[dword ptr _HL]\n");
					Write("al");
					}
				else
					printf("    mov [byte ptr _%s],al\n",Reg[r1]);
				}
			EndDefine();				/* Terminate the instruction */
			}
		}
	}

/* Generate C6-F6 & CE-FE, Arithmetic Operation A <op> Const */

for (i = 0;i < 8;i++)   				/* 8 math ops */
	{
	Define(G_STD,
		   (i / 2) * 0x10 + (i % 2) * 0x08 + 0xC6,
		   7,
		   "%s a,#nn",ALU[i]);
	Fetch("al");						/* Fetch the value */
	Operation(OP_ADD+i);				/* Do the operation */
	EndDefine();
	}

/* Generate 80-BE Arithmetic Operation A <op> Register */

for (i = 0;i < 8;i++)					/* 8 math ops */
	{
	for (r1 = 0;r1 < 8;r1++)			/* 8 registers */
		{
		Define(G_STD,					/* Define the instruction */
			   0x80+i*8+r1,
			   (r1 == 6 ? 7 : 4),
			   "%s a,%s",ALU[i],Reg[r1]);

		if (r1 == 6)					/* Get the other operand */
			{
			printf("    mov ebx,[dword ptr _HL]\n");
			Read("al");
			}
		else
			printf("    mov al,[byte ptr _%s]\n",Reg[r1]);

		Operation(OP_ADD+i);			/* Do the operation */
		EndDefine();
		}
	}

/* Generate 0x-3x x = 4,5,6,C,D,E inc r8,dec r8,ld r8,<const> */

for (r1 = 0;r1 < 8;r1++)				/* 8 registers */
	{
	Define(G_STD,						/* ld reg,constant */
		   r1*8+6,
		   (r1 == 6) ? 10 : 7,
		   "ld %s,#nn",Reg[r1]);
	Fetch("al");						/* Get the constant */
	if (r1 == 6)						/* and save it */
		{
		printf("    mov ebx,[dword ptr _HL]\n");
		Write("al");
		}
	else
		printf("    mov [byte ptr _%s],al\n",Reg[r1]);
	EndDefine();

	for (i = 0;i < 2;i++)				/* 0 = INC 1 = DEC */
		{
		Define(G_STD,					/* Define the appropriate inst */
			   r1*8+4+i,
			   (r1 == 6 ? 11 : 4),
			   "%s %s",
			   (i == 0 ? "inc" : "dec"),
			   Reg[r1]);

		if (r1 == 6)					/* Read the value */
			{                       	/* this is for (HL) */
			printf("    mov ebx,[dword ptr _HL]\n");
			Read("al");
			Operation(i == 0 ?			/* Do the operation */
						OP_INC8 : OP_DEC8);
			printf("    mov ebx,[dword ptr _HL]\n");
			Write("al");				/* Write it back */
			}
		else							/* This is for a real register */
			{
			printf("    mov al,[byte ptr _%s]\n",Reg[r1]);
			Operation(i == 0 ? OP_INC8 : OP_DEC8);
			printf("    mov [byte ptr _%s],al\n",Reg[r1]);
			}

		EndDefine();
		}
	}
GenerateIncDecTable();					/* Table of flags for INC & DEC */
}

/********************************************************************/
/*																	*/
/*			  Generate Flags Table for Inc & Dec (8 bit)			*/
/*																	*/
/********************************************************************/

static void GenerateIncDecTable()
{
int i,r;
Comment("Flag Tables for 8 bit Increment and Decrement");
printf("IncFlagTable:\n");
for (i = 0;i < 256;i++)
	{
	r = i & (0x80 + 0x20 + 0x08);		/* Get S, bit 5 and bit 3 */
	if (i == 0) r = r | 0x40;			/* Get Z bit */
	if ((i & 0xF) == 0) r = r | 0x10;	/* Get Half Carry Bit */
	if (i == 128) r = r | 0x04;			/* Get Parity/Overflow */
	printf("    db 0%02xh ; $%02x %s\n",r,i,IntToBin(r));
	}
printf("DecFlagTable:\n");
for (i = 0;i < 256;i++)
	{
	r = i & (0x80 + 0x20 + 0x08 + 0x02);/* Get S,N, bit 5 and bit 3 */
	if (i == 0) r = r | 0x40;			/* Get Z bit */
	if ((i & 0xF) == 0xF) r = r | 0x10;	/* Get Half Carry Bit */
	if (i == 127) r = r | 0x04;			/* Get Parity/Overflow */
	printf("    db 0%02xh ; $%02x %s\n",r,i,IntToBin(r));
	}
}