/********************************************************************/
/********************************************************************/
/*																	*/
/*					Code Generation File 3							*/
/*																	*/
/*	JP JR CALL RET RST												*/
/*																	*/
/********************************************************************/
/********************************************************************/

#include "z80.h"

/********************************************************************/
/*																	*/
/*						Z80 Code Generation							*/
/*																	*/
/*	CALL JP JR RET RST instructions									*/
/*																	*/
/********************************************************************/

static char *OpNames[] = { 	"call",	"jp",	"jr",	"ret" };
static int	 UnCond[]  = {  0xCD,   0xC3,	0x18,   0xC9  };
static int	 CondBase[]= {	0xC4,	0xC2,	0x20,	0xC0  };
static int	 DoTState[]= {  17,     10,     12,     11	  };
static int	 NoTState[]= {	10,		10,		7,		5	  };

static char *AdName[] = { "","nz ","z ","nc ","c ","po ","pe ","p ","m " };

static int   BitTest[] = { F_ZERO,F_CARRY,F_PARITY,F_SIGN };

void Gen3(void)
{
int Mode,Command,Op,t1,Label,n;
for (Command = 0;Command < 4;Command++)	/* There are 4 Commands */
	{
	for (Mode = -1;						/* There are 9 'modes' - uncond */
		 Mode < ((Command == 2) ? 4:8);	/* and the 8 conditionals */
		 Mode++)
		{
		Op = (Mode < 0) ?				/* Get the opcode */
				UnCond[Command] : CondBase[Command] + Mode * 8;

		t1 = (Mode < 0) ?				/* Get initial no of t states */
				DoTState[Command] :
				NoTState[Command];

		Define(G_STD,					/* Define the command */
			   Op,
			   t1,
			   "%s %s%s",
			   OpNames[Command],
			   AdName[Mode+1],
			   (Command != 3) ? "NNNN" : "");

		Label = Alloc(1);				/* Allocate a label if required */

										/* Fetch the argument, always done ! */
		if (Command == 0||Command == 1)	/* JP and CALL 16 bit */
								FetchWord("cx");
		if (Command == 2)				/* JR 8 bit */
								Fetch("cl");

		if (Mode >= 0)					/* Conditional ? */
			{                           /* if so test the appropriate bit */
			printf("    test [byte ptr _F],0%02xh\n",
				   BitTest[Mode/2]);
			printf("    j%s short %s\n",/* skip if appropriately set */
				   (Mode % 2 == 0) ? "nz" : "z",
				   SysLabel(Label));
			t1 = DoTState[Command]-		/* if conditional calculate extra */
					NoTState[Command];	/* t-states for the body & add */
			if (t1 != 0) AddCycles(t1);	/* them */
			}

		switch(Command)					/* Do the bulk */
			{
			case 0:						/* CALL */
				printf("    mov ax,[word ptr _PC]\n");
				printf("    mov [word ptr _PC],cx\n");
				Operation(OP_PUSH16);
				break;
			case 1:						/* JP (16 bit absolute) */
				printf("    mov [word ptr _PC],cx\n");
				break;
			case 2:						/* JR (8 bit relative) */
				printf("    movsx cx,cl\n");
				printf("    add [word ptr _PC],cx\n");
				break;
			case 3:						/* RET */
				Operation(OP_POP16);
				printf("    mov [word ptr _PC],ax\n");
				break;
			}

		if (Mode >= 0)					/* Conditional requires label */
			printf("%s:\n",SysLabel(Label));

		EndDefine();
		}
	}

for (n = 0;n < 8;n++)					/* RST xx instructions - short CALL */
	{
	Define(G_STD,n*8+0xC7,11,"rst %x",n*8);
	printf("    mov ax,[word ptr _PC]\n");
	Operation(OP_PUSH16);
	printf("    mov [word ptr _PC],0%02xh\n",n*8);
	EndDefine();
	}

}