/********************************************************************/
/********************************************************************/
/*																	*/
/*					Code Generation File 4							*/
/*																	*/
/********************************************************************/
/********************************************************************/

#include "z80.h"

/********************************************************************/
/*																	*/
/*						Z80 Code Generation							*/
/*																	*/
/*	Miscellaneous 'base' instructions								*/
/*																	*/
/********************************************************************/

void Gen4(void)
{
int n;
char *RotName[] = { "rlca","rrca","rla","rra" };

Define(G_STD,0x00,4,"nop");				/* 00 No-operation */
EndDefine();

Define(G_STD,0x08,4,"ex af,'af");		/* 08 Exchange AF and 'AF */
Exchange("_AF","_XAF");
EndDefine();

Define(G_STD,0x10,8,"djnz NNNN");		/* 10 Decrement B Jump if Non-Zero */
n = Alloc(1);
Fetch("al");
printf("    dec [byte ptr _B]\n");		/* Do the decrement of B */
printf("    jz short %s\n",SysLabel(n));/* If non-zero nothing else... */
printf("    movsx ax,al\n");			/* Calculate the offset */
printf("    add [word ptr _PC],ax\n");	/* Add to the PC */
AddCycles(5);
printf("%s:\n",SysLabel(n));
EndDefine();

Define(G_STD,0x22,16,"ld (NNNN),hl");	/* 22 Save HL in Memory */
FetchWord("bx");
printf("    mov ax,[word ptr _HL]\n");
WriteWord("ax");
EndDefine();

Define(G_STD,0x2A,16,"ld hl,(NNNN)");	/* 2A Load HL from Memory */
FetchWord("bx");
ReadWord("ax");
printf("    mov [word ptr _HL],ax\n");
EndDefine();

Define(G_STD,0x32,13,"ld (NNNN),a");	/* 32 Save A in Memory */
FetchWord("bx");
printf("    mov al,[byte ptr _A]\n");
Write("al");
EndDefine();

Define(G_STD,0x3A,13,"ld a,(NNNN)");	/* 3A Load A from Memory */
FetchWord("bx");
Read("al");
printf("    mov [byte ptr _A],al\n");
EndDefine();

Define(G_STD,0x37,4,"scf");				/* 37 Set Carry Flag */
n = 255-(F_SUM+F_HCARRY+F_COPY5+F_COPY3);
printf("    and [byte ptr _F],0%02xh ; %s\n",n,IntToBin(n));
printf("    or  [byte ptr _F],0%02xh ; %s\n",F_CARRY,IntToBin(F_CARRY));
#ifdef BIT35
Abort();
#endif
EndDefine();

Define(G_STD,0x3F,4,"ccf");				/* 3F Complement Carry Flag */
n = 255-(F_SUM+F_HCARRY+F_COPY5+F_COPY3);
printf("    and [byte ptr _F],0%02xh ; %s\n",n,IntToBin(n));
printf("    xor [byte ptr _F],0%02xh ; %s\n",F_CARRY,IntToBin(F_CARRY));
#ifdef BIT35
Abort();
#endif
EndDefine();

Define(G_STD,0x2F,4,"cpl");				/* 2F 1's complement Accumulator */
printf("    xor [byte ptr _A],0FFh\n");
n = 255-(F_COPY5+F_COPY3);
printf("    and [byte ptr _F],0%02xh ; %s\n",n,IntToBin(n));
n = F_HCARRY+F_SUM;
printf("    or [byte ptr _F],0%02xh ; %s\n",n,IntToBin(n));
#ifdef BIT35
Abort();
#endif
EndDefine();

Define(G_STD,0x27,4,"daa");				/* 27 Decimal Adjust A */
n = Alloc(2);
printf("    mov al,[byte ptr _A]\n");	/* Get the value to adjust */
printf("    mov  ah,[byte ptr _F]\n");	/* and the flags */
printf("    test ah,0%02xh ; %s\n",		/* if ADD or SUB .... */
					F_SUM,IntToBin(F_SUM));
printf("    jnz short %s\n",SysLabel(n));
printf("    sahf\n");					/* This is Decimal ADD */
printf("    daa\n");
printf("    jmp short %s\n",SysLabel(n+1));
printf("%s:\n",SysLabel(n));    		/* This is Decimal SUB */
printf("    sahf\n");
printf("    das\n");
printf("%s:\n",SysLabel(n+1));
SaveStatus(SS_NORMAL);					/* Update the flags - NOT overflow */
printf("    mov [byte ptr _A],al\n");	/* save the result */
EndDefine();

Define(G_STD,0xD9,4,"exx");				/* D9 Exchange Registers */
Exchange("_BC","_XBC");
Exchange("_DE","_XDE");
Exchange("_HL","_XHL");
EndDefine();

Define(G_STD,0xEB,4,"ex de,hl");		/* EB Exchange DE & HL */
Exchange("_DE","_HL");
EndDefine();

Define(G_STD,0xE9,4,"jp (hl)");			/* E9 PC = HL */
printf("    mov ax,[word ptr _hl]\n");
printf("    mov [word ptr _pc],ax\n");
EndDefine();

Define(G_STD,0xF9,6,"ld sp,hl");		/* F9 SP = HL */
printf("    mov ax,[word ptr _hl]\n");
printf("    mov [word ptr _sp],ax\n");
EndDefine();

Define(G_STD,0xFB,4,"ei");				/* FB Enable Interrupts */
printf("    mov [byte ptr _IFF2],1\n");
printf("    mov [byte ptr _IFF1],1\n");
EndDefine();

Define(G_STD,0xF3,4,"di");				/* F3 Disable Interrupts */
printf("    mov [byte ptr _IFF2],0\n");
printf("    mov [byte ptr _IFF1],0\n");
EndDefine();

Define(G_STD,0xE3,19,"ex (sp),hl");		/* E3 exchange TOS with HL */
printf("    mov ebx,[dword ptr _SP]\n");
ReadWord("ax");
printf("    xchg ax,[word ptr _HL]\n");
printf("    mov ebx,[dword ptr _SP]\n");
WriteWord("ax");
EndDefine();

Define(G_STD,0xD3,11,"out (nn),a");		/* D3 Output to port */
Fetch("al");
printf("    mov ah,[byte ptr _A]\n");
printf("    call IOutPort\n");
EndDefine();

Define(G_STD,0xDB,11,"in a,(nn)");		/* DB Input from port */
Fetch("al");
printf("    mov ah,[byte ptr _A]\n");	/* High bytes from A */
printf("    call IInPort\n");			/* in a,(nn) sets no flags */
printf("    mov [byte ptr _A],al\n");
EndDefine();

for (n = 0;n < 4;n++)					/* The 4 base rotates which don't */
	{									/* set parity,zero or sign */
	Define(G_STD,0x7+n*8,4,RotName[n]);
	printf("    mov al,[byte ptr _A]\n");
	Operation( (OP_RLC+n) | OPX_NOSZP );
	printf("    mov [byte ptr _A],al\n");
	EndDefine();
	}

for (n = 0;n < 2;n++)					/* DD and FD Extend */
	{
	Define(G_STD,0xDD+n*0x20,0,"extend %02x",n*0x20+0xDD);
	printf("    mov esi,offset _I%c\n",n+'X');
	Dispatcher(G_xD,TRUE);
	EndDefine();
	}

Define(G_STD,0x76,4,"halt");			/* 76 Halt */
printf("    sub [word ptr _PC],1\n");	/* stop ... an interrupt will wake */
EndDefine();							/* us up */

Define(G_STD,0xED,0,"extend ed");		/* ED extend */
Dispatcher(G_ED,TRUE);
EndDefine();

Define(G_STD,0xCB,0,"extend cb");		/* CB extend */
Dispatcher(G_CB,TRUE);
EndDefine();

}

void Exchange(char *r1,char *r2)
{
printf("    mov ax,[word ptr %s]\n",r1);
printf("    xchg ax,[word ptr %s]\n",r2);
printf("    mov [word ptr %s],ax\n",r1);
}