/********************************************************************/
/********************************************************************/
/*																	*/
/*					Code Generation File 5							*/
/*																	*/
/*	Basic IX and IY instructions (not the CB extended ones !)		*/
/*																	*/
/*	Note : For IX IY (e.g. DD FD instructions) on entry ESI will	*/
/*		   point to the register. There is only one instruction for */
/*		   each 'pair'.												*/
/*																	*/
/********************************************************************/
/********************************************************************/

#include "z80.h"

#define ISHLX(n) ((n) >= 4 && (n) <= 6)

static char *ALU[] = { "add","adc","sub","sbc","and","xor","or","cp" };

/********************************************************************/
/*																	*/
/*						Z80 Code Generation							*/
/*																	*/
/********************************************************************/

void Gen5(void)
{
int n,n1,r,rt,rs,rs1,rt1;
static char *AddReg[] = { "bc","de","ixy","sp" };
static char *Reg8[] = { "b","c","d","e","ixyh","ixyl","(ixy+rr)","a","h","l" };

Define(G_xD,0x21,14,"ld ixy,#nnnn");	/* $21 load ixy with constant */
FetchWord("ax");
printf("    mov [esi],ax\n");
EndDefine();

Define(G_xD,0x22,20,"ld (nnnn),ixy");	/* $22 save ixy in memory */
FetchWord("bx");
printf("    mov ax,[esi]\n");
WriteWord("ax");
EndDefine();

Define(G_xD,0x23,10,"inc ixy");			/* $23 increment ixy */
printf("    inc [word ptr 0+esi]\n");
EndDefine();

Define(G_xD,0x2A,20,"ld ixy,(nnnn)");	/* $2A load ixy from memory */
FetchWord("bx");
ReadWord("ax");
printf("    mov [esi],ax\n");
EndDefine();

Define(G_xD,0x2B,10,"dec ixy");			/* $2B decrement ixy */
printf("    dec [word ptr 0+esi]\n");
EndDefine();

Define(G_xD,0xE1,14,"pop ixy");			/* $E1 pop ix */
Operation(OP_POP16);
printf("    mov [esi],ax\n");
EndDefine();

Define(G_xD,0xE3,23,"ex (sp),ixy");		/* $E3 ex (sp),ixy */
printf("    mov ebx,[dword ptr _SP]\n");
ReadWord("ax");
printf("    xchg ax,[esi]\n");
printf("    mov ebx,[dword ptr _SP]\n");
WriteWord("ax");
EndDefine();

Define(G_xD,0xE5,15,"push ixy");		/* $E5 push ixy */
printf("    mov ax,[esi]\n");
Operation(OP_PUSH16);
EndDefine();

Define(G_xD,0xE9,15,"jp (ixy)");		/* $E9 jp (ixy) */
printf("    mov ax,[esi]\n");
printf("    mov [word ptr _PC],ax\n");
EndDefine();

Define(G_xD,0xF9,10,"ld sp,ixy");		/* $F9 ld sp,ixy */
printf("    mov ax,[esi]\n");
printf("    mov [word ptr _SP],ax\n");
EndDefine();

Define(G_xD,0xCD,0,"call xxxx");		/* $DD CB call for 'Batman' */
printf("   call %s\n",OpLabel(G_STD,0xCD));
EndDefine();

for (n = 0;n < 4;n++)					/* $09-$39 add ixy bc,de,ixy,sp */
	{
	Define(G_xD,0x09+n*0x10,15,"add ixy,%s",AddReg[n]);
	printf("    mov ax,[esi]\n");
	printf("    mov bx,[%s%s]\n",
				(n == 2) ? "esi" : "word ptr _",
				(n == 2) ? "" : AddReg[n]);
	Operation(OP_ADD16);
	printf("    mov [esi],ax\n");
	EndDefine();
	}

for (n = 4;n < 7;n++)					/* IXYL IXYH and (IXY+rr) */
	{
	for (n1 = 0;n1 < 2;n1++)    		/* INC and DEC */
		{
		Define(G_xD,					/* Start the instruction */
			   0x24+(n-4)*8+n1,
			   (n == 6) ? 23 : 8,
			   "%s %s",(n1 == 0 ? "inc" : "dec"),Reg8[n]);

		if (n != 6)						/* Get value - this is IXH and IXL */
			printf("    mov al,[esi+%d]\n",5-n);
		else
			{							/* If INC (IX+n) then calc & save */
			EACIndex();					/* address */
			printf("    push ebx\n");
			Read("al");
			}

		Operation(n1 == 0 ?				/* Do it ! */
						OP_INC8 : OP_DEC8);

		if (n != 6)						/* Write back IXL IXH */
			printf("    mov [esi+%d],al\n",5-n);
		else
			{                           /* INC (IXY+n) */
			printf("    pop ebx\n");	/* Restore address & write */
			Write("al");
			}

		EndDefine();
		}

	Define(G_xD,0x26+(n-4)*8,			/* Load constants LD xxx,n */
				(n == 6) ? 19 : 11,"ld %s,nn",Reg8[n]);
	if (n == 6)							/* If ld (ixy+n) calculate EAC */
		{								/* and save it */
		EACIndex();
		printf("    push ebx\n");
		}
	Fetch("al");						/* Get value */
	if (n == 6)							/* ld (ixy+r),n */
		{
		printf("    pop ebx\n");		/* Restore address & write */
		Write("al");
		}
	else								/* Save to appropriate register */
		printf("    mov [esi+%d],al\n",5-n);
	EndDefine();
	}

for (rt = 0;rt < 8;rt++)				/* Register to Register transfers */
	for (rs = 0;rs < 8;rs++)			/* LD r,r where r is b,c,d,e,ixyh */
		{								/* ixyl (ixy+n) a EXCEPT ... */
		if (rs != rt && (ISHLX(rs) || ISHLX(rt)))
			{
			rs1 = rs;rt1 = rt;			/* Fix if (IXY+nn) on one side */
			if (rs == 6 &&				/* use H and L rather than IXYHL */
				(rt == 4 || rt == 5)) rt1 = rt1+4;
			if (rt == 6 &&
				(rs == 4 || rs == 5)) rs1 = rs1+4;

			Define(G_xD,				/* Start the opcode */
				   0x40+rt*8+rs,
				   (rs == 6 || rt == 6) ? 19 : 8,
				   "ld %s,%s",Reg8[rt1],Reg8[rs1]);

			if (rs == 6 || rt == 6)		/* EAC for (IXY+n) if one is that */
								EACIndex();

			switch(rs1)         		/* Get the source */
				{
				case 4:					/* IXYH and IXYL */
				case 5:
					printf("    mov al,[esi+%d]\n",5-rs1);break;
				case 6:					/* (IXY+r), calculated earlier */
					Read("al");break;
				default:				/* any other register */
					printf("    mov al,[byte ptr _%s]\n",Reg8[rs1]);
				}

			switch(rt1)					/* Save the answer */
				{
				case 4:					/* IXYH and IXYL */
				case 5:
					printf("    mov [esi+%d],al\n",5-rt1);break;
				case 6:					/* (IXY+n) calculated earlier */
					Write("al");break;
				default:				/* any other register */
					printf("    mov [byte ptr _%s],al\n",Reg8[rt1]);
				}

			EndDefine();
			}
		}

for (r = 4; r < 7;r++)					/* Math ops. Only IXL IXH and (IXY+n) */
	for (n = 0;n < 8;n++)				/* for all operations */
		{
		Define(G_xD,					/* Instruction */
			   0x80+n*8+r,
			   (r == 6) ? 19 : 8,
			   "%s a,%s",ALU[n],Reg8[r]);

		if (r == 6)						/* Fetch the other operand */
			{
			EACIndex();
			Read("al");
			}
		else printf("    mov al,[esi+%d]\n",5-r);
		Operation(OP_ADD+n);			/* Do the ALU op */

		EndDefine();
		}

for (n = 0;n < 2;n++)					/* Extend DD and FD can be repeated */
	{
	Define(G_xD,0xDD+n*0x20,0,"extend %02x",n*0x20+0xDD);
	printf("    mov esi,offset _I%c\n",n+'X');
	Dispatcher(G_xD,TRUE);
	EndDefine();
	}

printf("_DCBAddr dd 0\n");				/* Temporary location */

Define(G_xD,0xCB,0,"extend CB");		/* Extend CB can be repeated */
EACIndex();								/* get the offset & address NOW */
printf("    mov [_DCBAddr],ebx\n");		/* Save it for later */
Dispatcher(G_xDCB,FALSE);				/* Doesn't bump R again */
EndDefine();
}

void EACIndex(void)
{
Fetch("al");
printf("    movsx bx,al\n");
printf("    add bx,[esi]\n");
}