/********************************************************************/
/********************************************************************/
/*																	*/
/*						Operations on Data		 					*/
/*																	*/
/*	Generates all the repetitive code .... math, logic, stack ops.. */
/*																	*/
/********************************************************************/
/********************************************************************/

#include "z80.h"

static void LoadCarryFlag(void);
static void RotateAL(int Op,BOOL bNoZSP);

/********************************************************************/
/*																	*/
/*					    Generate Operation Code 					*/
/*																	*/
/********************************************************************/

void Operation(int Op)
{
char *Func;
BOOL IsAdd;
int n;
switch(Op & 0xFFF)
	{
	case OP_PUSH16:						/*** Push AX on Z80 Stack ***/
		printf("    sub [word ptr _SP],2            ; push AX on Z80 Stack\n");
		printf("    mov ebx,[dword ptr _SP]\n");
		WriteWord("ax");
		break;

	case OP_POP16:						/*** Pop AX off Z80 Stack ***/
		printf("    mov ebx,[dword ptr _SP]         ; pop AX off Z80 Stack\n");
		ReadWord("ax");
		printf("    add [word ptr _SP],2\n");
		break;

	case OP_ADD:						/*** Add AL to A (w/wo carry) ***/
	case OP_ADC:
		if (Op == OP_ADC) LoadCarryFlag();
		printf("    %s [byte ptr _A],al\n",
				(Op == OP_ADC ? "adc" : "add"));
		SaveStatus(SS_CLEARN+SS_OVERFLOW);
		#ifdef BIT35					/* Code for setting 3&5 of F */
		ERROR();
		#endif
		break;

	case OP_SUB:						/*** Sub AL from A (w/wo borrow) ***/
	case OP_SBC:
		if (Op == OP_SBC) LoadCarryFlag();
		printf("    %s [byte ptr _A],al\n",
				(Op == OP_SBC ? "sbb" : "sub"));
		SaveStatus(SS_SETN+SS_OVERFLOW);
		#ifdef BIT35					/* Code for setting 3&5 of F */
		ERROR();
		#endif
		break;

	case OP_CP:							/*** Compare AL against A ***/
		printf("    cmp [byte ptr _A],al\n");
		SaveStatus(SS_SETN+SS_OVERFLOW);
		#ifdef BIT35					/* Code for setting 3&5 of F */
		ERROR();
		#endif
		break;

	case OP_AND:						/*** A <logical op> AL ***/
	case OP_OR:
	case OP_XOR:
		Func = "xor";					/* Decide what to be used */
		if (Op == OP_AND) Func = "and";
		if (Op == OP_OR) Func = "or";
		printf("    %s [_A],al\n",Func);/* Do the operation */
		SaveStatus(SS_CLEARNC);			/* Save the status */
		#ifdef BIT35					/* Code for setting 3&5 of F */
		ERROR();
		#endif
		break;

	case OP_INC8:						/*** 8 bit INC and DEC ***/
	case OP_DEC8:
		printf("    %s al\n",(Op == OP_INC8 ? "inc":"dec"));
		printf("    and eax,0FFh\n");
		printf("    mov ah,[byte ptr %sFlagTable+eax]\n",
			   (Op == OP_INC8 ? "Inc" : "Dec"));
		printf("    and [byte ptr _F],0%02xh\n",F_CARRY);
		printf("    or  [byte ptr _F],ah\n");
		break;

	case OP_RLC:
	case OP_RL:
	case OP_RRC:
	case OP_RR:
	case OP_SLA:
	case OP_SLL:
	case OP_SRA:
	case OP_SRL:
		RotateAL(Op & 0xFFF,(Op & OPX_NOSZP) != 0);
		break;

	case OP_ADD16:						/*** 16 bit addition ***/
		printf("    add ax,bx\n");
		printf("    push eax\n");
		SaveStatus(SS_CLEARN | SS_NOSIGN | SS_NOZERO | SS_NOPARITY);
		#ifdef BIT35					/* Code for setting 3&5 of F */
		ERROR();
		#endif
		printf("    pop eax\n");
		break;

	case OP_ADC16:						/*** 16 bit carry addition/sub */
	case OP_SBC16:
		IsAdd = ((Op & 0xFFF) == OP_ADC16);

		printf("    mov cx,ax\n");		/* save the first operand */
		LoadCarryFlag();				/* set carry accordingly */
		printf("    mov ax,cx\n");		/* Get the first operand back */
		printf("    pushf\n");			/* save the flag register */

		printf("    %s ax,bx\n",		/* calculate the answer */
					(IsAdd ? "adc":"sbb"));
		printf("    mov dx,ax\n");		/* save it in DX */

		SaveStatus( (IsAdd ?			/* Update 'F' */
						SS_CLEARN : SS_SETN));

		n = 255-F_HCARRY-F_PARITY;		/* Zero the hcarry / parity bits */
		printf("    and [byte ptr _F],0%02xh ; %s\n",n,IntToBin(n));
		printf("    popf\n");			/* Restore the original carry to F */

										/* Recalculate it 8 bit style */
		printf("    %s cl,bl\n",		/* to get the real Half Carry */
				(IsAdd ? "adc":"sbb"));	/* and overflow flag values */
		printf("    %s ch,bh\n",
				(IsAdd ? "adc":"sbb"));
		printf("    lahf\n");       	/* save PSW for half-carry */
		n = Alloc(1);					/* do the overflow test */
		printf("    jno %s\n",SysLabel(n));
		printf("    or [byte ptr _F],0%02xh ; %s\n",
								F_PARITY,IntToBin(F_PARITY));
		printf("%s:\n",SysLabel(n));
										/* now update the half carry */
		printf("    and ah,0%02xh ; %s\n",F_HCARRY,IntToBin(F_HCARRY));
		printf("    or [byte ptr _F],ah\n");

		#ifdef BIT35					/* Code for setting 3&5 of F */
		ERROR();
		#endif

		printf("    mov ax,dx\n");		/* Get the result back */
		break;

	default:
		ERROR();
		break;
	}
}

/********************************************************************/
/*																	*/
/*				Load Carry Flag from _F into PSW					*/
/*																	*/
/********************************************************************/

static void LoadCarryFlag(void)
{
printf("    mov ah,[byte ptr _F]            ; copy _F to carry flag\n");
printf("    sahf\n");
}

/********************************************************************/
/*																	*/
/*						Save the PSW into _F            			*/
/*																	*/
/*	Default actions saves S Z H P and C 							*/
/*																	*/
/*	SS_SETN/SS_CLEARN	Set the 'N' flag accordingly				*/
/*	SS_OVERFLOW			Copies overflow into 'P'					*/
/*	SS_NOxxxx			Don't copy given flag						*/
/*																	*/
/********************************************************************/

void SaveStatus(int Flags)
{
int i,Mask,OldMask;

printf("    lahf\n");					/* Save the flags */
Mask = F_SIGN+F_ZERO+F_HCARRY+			/* We want Sign,Zero,Parity,Carry */
					F_PARITY+F_CARRY;	/* Half-Carry,from AH by Default */

if (Flags & SS_NOSIGN) Mask &= (255-F_SIGN);
if (Flags & SS_NOZERO) Mask &= (255-F_ZERO);
if (Flags & SS_NOPARITY) Mask &= (255-F_PARITY);
if (Flags & SS_NOCARRY) Mask &= (255-F_CARRY);

OldMask = 255-Mask;						/* Bits we want from the old Flag */
										/* Register */
OldMask = OldMask-(F_COPY3+F_COPY5);	/* Clear Bits 3 & 5 too */

if (Flags & SS_CLEARN)					/* If clearing N that is zero */
	{
	OldMask = OldMask & (255-F_SUM);	/* Clear bit in both parts */
	Mask = Mask & (255-F_SUM);
	}

if (Flags & SS_CLEARP)					/* If clearing P that is zero */
	{
	OldMask = OldMask & (255-F_PARITY);	/* Clear bit in both parts */
	Mask = Mask & (255-F_PARITY);
	}

if (Flags & SS_CLEARH)					/* Half carry is zero */
	{
	OldMask = OldMask & (255-F_HCARRY);	/* Clear bit in both parts */
	Mask = Mask & (255-F_HCARRY);
	}

if (Flags & SS_CLEARNC)					/* If clearing N AND C */
	{
	OldMask = OldMask & (255-F_CARRY-F_SUM);
	Mask = Mask & (255-F_CARRY-F_SUM);
	}

if (Flags & SS_OVERFLOW)				/* Copy overflow to Parity */
	{
	i = Alloc(2);
	printf("    jo short %s             ; copy overflow to parity\n",SysLabel(i));
	printf("    and ah,0%02xh\n",255-F_PARITY);
	printf("    jmp short %s\n",SysLabel(i+1));
	printf("%s:\n",SysLabel(i));
	printf("    or ah,0%02xh\n",F_PARITY);
	printf("%s:\n",SysLabel(i+1));
	}

if (Flags & SS_SETN)					/* if setting N flag */
	{
	Mask = Mask | F_SUM;				/* Don't mask it out ! */
	OldMask = OldMask & (255-F_SUM);	/* Can be masked in old if wanted */
    									/* Set the bit in the status reg */
	printf("    or ah,%d                         ; set N bit (0x02)\n",F_SUM);
	}

if (OldMask == 0)						/* Keeping nothing of old F reg */
	{									/* Chop the bits off to zero */
	printf("    and ah,0%02xh                     ; %s\n",
		   Mask,
		   IntToBin(Mask));
	printf("    mov  [_F],ah\n");		/* Save the result */
	}

else									/* Keeping some of old F */
	{									/* Keep the bits we want */
	printf("    and [byte ptr _F],0%02xh          ; %s\n",
		   OldMask,
		   IntToBin(OldMask));			/* Sort out the new bits */
	printf("    and ah,0%02xh                     ; %s\n",
		   Mask,
		   IntToBin(Mask));
	printf("    or  [_F],ah\n");		/* Merge them */
	}

}

/********************************************************************/
/*																	*/
/*				Convert Flag value to text display					*/
/*																	*/
/********************************************************************/

char *IntToBin(int Value)
{
int i;
static char ITBBuffer[9];
strcpy(ITBBuffer,"________");
for (i = 0;i < 8;i++)
	{
	if (Value & (128 >> i)) ITBBuffer[i] = ("SZXHXPNC")[i];
	}
return(ITBBuffer);
}

/********************************************************************/
/*																	*/
/*				Rotoate Flag, saving according to type				*/
/*					  (bNoZSP is TRUE if RLCA RLA)					*/
/*																	*/
/********************************************************************/

static void RotateAL(int Op,BOOL bNoZSP)
{
int Flag;

switch(Op & 0xFFF)
	{
	case OP_RL:        					/* Rotate left through carry */
		LoadCarryFlag();
		printf("    rcl al,1\n");
		break;
	case OP_RR:							/* Rotate right through carry */
		LoadCarryFlag();
		printf("    rcr al,1\n");
		break;
	case OP_RLC:						/* Rotate left into carry */
		printf("    rol al,1\n");
		break;
	case OP_RRC:						/* Rotate right into carry */
		printf("    ror al,1\n");
		break;

	case OP_SLA:						/* Shift left into carry */
		printf("    add al,al\n");
		break;

	case OP_SRA:						/* Shift right into carry */
		printf("    sar al,1\n");		/* (arithmetic) */
		break;

	case OP_SLL:						/* Shift left logical */
		printf("    stc\n");
		printf("    adc al,al\n");
		break;

	case OP_SRL:						/* Shift right logical */
		printf("    shr al,1\n");
		break;

	default:
		ERROR();break;
	}


printf("    lahf\n");					/* First we get the carry out */

Flag = F_COPY3 + F_COPY5 +				/* All these are set to zero */
					F_SUM + F_HCARRY + F_CARRY;
Flag = 255-Flag;						/* Make it into a mask */

printf("    and [byte ptr _F],0%02xh ; %s\n",
								Flag,IntToBin(Flag));
printf("    and ah,0%02xh ; %s\n",		/* Isolate the carry */
						F_CARRY,IntToBin(F_CARRY));
printf("    or  [byte ptr _F],ah\n");	/* Mix into the flag register */

if (!bNoZSP)							/* Are we setting Z S and P ? */
	{
	printf("    or al,al\n");			/* Get it */
	SaveStatus(SS_CLEARN + SS_CLEARH + SS_NOCARRY);
	}

#ifdef BIT35							/* Code for setting 3&5 of F */
ERROR();
#endif

}