/********************************************************************/
/********************************************************************/
/*																	*/
/*						Z80 General Include File					*/
/*						------------------------					*/
/*																	*/
/********************************************************************/
/********************************************************************/

/*#define BIT35*/						/* Bit 3 & 5 of flag check */

#include	<stdio.h> 					/* Include what we'll need */
#include 	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#define		G_STD		(0)				/* Standard Z80 Instructions */
#define 	G_CB		(1)				/* CB xx Z80 Instructions */
#define		G_ED		(2)				/* ED xx Z80 Instructions */
#define		G_xD		(3)				/* DD/FD xx Z80 Instructions */
#define		G_xDCB		(4)				/* DD/FD CB xx Z80 Instructions */

#define		SECTIONS	(5)				/* Number of Instruction Groups */
#define		FALSE		(0)				/* Boolean types */
#define		TRUE		(!(FALSE))
typedef		int			BOOL;

#define		ERROR()		Abort(__LINE__,__FILE__)
#define		ASSERT(x)	if (!(x)) ERROR()

#define		ISGROUP(n)	((n) >= 0 && (n) <= SECTIONS)
#define 	ISSTRING(s)	((s) != NULL && *(s) != '\0')

void		Fetch(char *Reg86);			/* Atomic Generation Functions */
void		FetchWord(char *Reg86);
void		Read(char *Reg86);
void		ReadWord(char *Reg86);
void		Write(char *Reg86);
void		WriteWord(char *Reg86);

int			Alloc(int Labels);			/* Utility Functions */
char		*SysLabel(int Label);
char		*OpLabel(int Group,int Op);
void		Define(int Group,int Op,int Cycles,char *Format,...);
void        EndDefine(void);
void		Abort(int Line,char *File);
BOOL		InString(char *String,char *List);
void		AddCycles(int Cycles);
void		Comment(char *Comment);
void 		GenerateJumpTables(void);
void		Dispatcher(int Table,BOOL bIncR);

void 		Gen1(void);					/* Generation functions */
void		Gen2(void);
void		Gen3(void);
void		Gen4(void);
void		Gen5(void);
void		Gen6(void);
void		Gen7(void);
void		Gen8(void);
void		Gen9(void);

void		Exchange(char *r1,char *r2);
void	  	EACIndex(void);

void		Operation(int Function);	/* Do a function */
void		SaveStatus(int Flags);		/* Save Status Flags */
char		*IntToBin(int Value);		/* F register as SZxHxPNC */

#define		OP_PUSH16		(0x101)		/* Push AX onto Z80 Stack */
#define		OP_POP16		(0x102)		/* Pop AX off Z80 Stack */
#define		OP_ADD			(0x200)		/* Add AL to _A. no carry */
#define		OP_ADC			(0x201)		/* Add AL to _A. carry */
#define		OP_SUB			(0x202)		/* Sub AL to _A. no carry */
#define		OP_SBC			(0x203)		/* Sub AL to _A. carry */
#define		OP_AND			(0x204)		/* And AL with _A */
#define		OP_XOR			(0x205)		/* Xor AL with _A */
#define		OP_OR 			(0x206)		/* Or  AL with _A */
#define		OP_CP 			(0x207)		/* Compare AL with _A */
#define		OP_INC8			(0x301)		/* 8 bit increment of AL */
#define 	OP_DEC8			(0x302)		/* 8 bit decrement of AL */
#define		OP_ADD16		(0x401)		/* 16 bit add of AX and BX */
#define		OP_ADC16		(0x402)		/* 16 bit adc of AX,BX */
#define		OP_SBC16		(0x403)		/* 16 bit sbc of AX,BX */
#define		OP_RLC			(0x501)		/* 8 bit rotate left */
#define		OP_RRC			(0x502)		/* 8 bit rotate right */
#define		OP_RL			(0x503)		/* 9 bit rotate left */
#define		OP_RR			(0x504)		/* 9 bit rotate right */
#define		OP_SLA			(0x505)		/* Shift left arithmetic */
#define		OP_SRA			(0x506)		/* Shift right arithmetic */
#define		OP_SLL			(0x507)		/* shift left logical */
#define		OP_SRL			(0x508)		/* shift right logical */

#define		OPX_NOSZP		(0x8000)	/* Don't set Z,N and P in rotate */

#define		SS_CLEARN		(0x4000)	/* Clear N Flag */
#define		SS_OVERFLOW		(0x2000)	/* Copy Overflow to Parity */
#define		SS_SETN			(0x1000)	/* Set N Flag */
#define		SS_CLEARNC		(0x0800)	/* Clear N and C */
#define 	SS_NOZERO		(0x0400)	/* Don't set zero */
#define		SS_NOSIGN		(0x0200)	/* Don't set sign bit */
#define		SS_NOPARITY		(0x0100)	/* Don't set parity bit */
#define		SS_CLEARH  		(0x0080)	/* Clear the half carry bit */
#define		SS_NOCARRY		(0x0040)	/* Don't set carry bit */
#define		SS_CLEARP  		(0x0020)	/* Clear the parity bit */
#define		SS_NORMAL		(0x0000)	/* Copy SZPHC clear bit 3 & 5 */

#define		F_SIGN			(0x80)		/* Sign Bit */
#define		F_ZERO			(0x40)		/* Zero Bit */
#define		F_COPY5			(0x20)		/* Copied bit */
#define		F_HCARRY		(0x10)		/* Half Carry bit */
#define		F_COPY3			(0x08)		/* Copied bit */
#define		F_PARITY		(0x04)		/* Parity/Overflow */
#define		F_SUM			(0x02)		/* Sum Direction Bit */
#define		F_CARRY			(0x01)		/* Carry bit */